# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Policy(pulumi.CustomResource):
    boolean_policy: pulumi.Output[dict]
    """
    A boolean policy is a constraint that is either enforced or not. Structure is documented below.

      * `enforced` (`bool`) - If true, then the Policy is enforced. If false, then any configuration is acceptable.
    """
    constraint: pulumi.Output[str]
    """
    The name of the Constraint the Policy is configuring, for example, `serviceuser.services`. Check out the [complete list of available constraints](https://cloud.google.com/resource-manager/docs/organization-policy/understanding-constraints#available_constraints).
    """
    etag: pulumi.Output[str]
    """
    (Computed) The etag of the organization policy. `etag` is used for optimistic concurrency control as a way to help prevent simultaneous updates of a policy from overwriting each other.
    """
    list_policy: pulumi.Output[dict]
    """
    A policy that can define specific values that are allowed or denied for the given constraint. It can also be used to allow or deny all values. Structure is documented below.

      * `allow` (`dict`) - or `deny` - (Optional) One or the other must be set.
        * `all` (`bool`) - The policy allows or denies all values.
        * `values` (`list`) - The policy can define specific values that are allowed or denied.

      * `deny` (`dict`)
        * `all` (`bool`) - The policy allows or denies all values.
        * `values` (`list`) - The policy can define specific values that are allowed or denied.

      * `inheritFromParent` (`bool`) - If set to true, the values from the effective Policy of the parent resource
        are inherited, meaning the values set in this Policy are added to the values inherited up the hierarchy.
      * `suggestedValue` (`str`) - The Google Cloud Console will try to default to a configuration that matches the value specified in this field.
    """
    org_id: pulumi.Output[str]
    """
    The numeric ID of the organization to set the policy for.
    """
    restore_policy: pulumi.Output[dict]
    """
    A restore policy is a constraint to restore the default policy. Structure is documented below.

      * `default` (`bool`) - May only be set to true. If set, then the default Policy is restored.
    """
    update_time: pulumi.Output[str]
    """
    (Computed) The timestamp in RFC3339 UTC "Zulu" format, accurate to nanoseconds, representing when the variable was last updated. Example: "2016-10-09T12:33:37.578138407Z".
    """
    version: pulumi.Output[float]
    """
    Version of the Policy. Default version is 0.
    """
    def __init__(__self__, resource_name, opts=None, boolean_policy=None, constraint=None, list_policy=None, org_id=None, restore_policy=None, version=None, __props__=None, __name__=None, __opts__=None):
        """
        Allows management of Organization policies for a Google Organization. For more information see
        [the official
        documentation](https://cloud.google.com/resource-manager/docs/organization-policy/overview) and
        [API](https://cloud.google.com/resource-manager/reference/rest/v1/organizations/setOrgPolicy).

        ## Example Usage



        ```python
        import pulumi
        import pulumi_gcp as gcp

        serial_port_policy = gcp.organizations.Policy("serialPortPolicy",
            boolean_policy={
                "enforced": True,
            },
            constraint="compute.disableSerialPortAccess",
            org_id="123456789")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] boolean_policy: A boolean policy is a constraint that is either enforced or not. Structure is documented below.
        :param pulumi.Input[str] constraint: The name of the Constraint the Policy is configuring, for example, `serviceuser.services`. Check out the [complete list of available constraints](https://cloud.google.com/resource-manager/docs/organization-policy/understanding-constraints#available_constraints).
        :param pulumi.Input[dict] list_policy: A policy that can define specific values that are allowed or denied for the given constraint. It can also be used to allow or deny all values. Structure is documented below.
        :param pulumi.Input[str] org_id: The numeric ID of the organization to set the policy for.
        :param pulumi.Input[dict] restore_policy: A restore policy is a constraint to restore the default policy. Structure is documented below.
        :param pulumi.Input[float] version: Version of the Policy. Default version is 0.

        The **boolean_policy** object supports the following:

          * `enforced` (`pulumi.Input[bool]`) - If true, then the Policy is enforced. If false, then any configuration is acceptable.

        The **list_policy** object supports the following:

          * `allow` (`pulumi.Input[dict]`) - or `deny` - (Optional) One or the other must be set.
            * `all` (`pulumi.Input[bool]`) - The policy allows or denies all values.
            * `values` (`pulumi.Input[list]`) - The policy can define specific values that are allowed or denied.

          * `deny` (`pulumi.Input[dict]`)
            * `all` (`pulumi.Input[bool]`) - The policy allows or denies all values.
            * `values` (`pulumi.Input[list]`) - The policy can define specific values that are allowed or denied.

          * `inheritFromParent` (`pulumi.Input[bool]`) - If set to true, the values from the effective Policy of the parent resource
            are inherited, meaning the values set in this Policy are added to the values inherited up the hierarchy.
          * `suggestedValue` (`pulumi.Input[str]`) - The Google Cloud Console will try to default to a configuration that matches the value specified in this field.

        The **restore_policy** object supports the following:

          * `default` (`pulumi.Input[bool]`) - May only be set to true. If set, then the default Policy is restored.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['boolean_policy'] = boolean_policy
            if constraint is None:
                raise TypeError("Missing required property 'constraint'")
            __props__['constraint'] = constraint
            __props__['list_policy'] = list_policy
            if org_id is None:
                raise TypeError("Missing required property 'org_id'")
            __props__['org_id'] = org_id
            __props__['restore_policy'] = restore_policy
            __props__['version'] = version
            __props__['etag'] = None
            __props__['update_time'] = None
        super(Policy, __self__).__init__(
            'gcp:organizations/policy:Policy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, boolean_policy=None, constraint=None, etag=None, list_policy=None, org_id=None, restore_policy=None, update_time=None, version=None):
        """
        Get an existing Policy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] boolean_policy: A boolean policy is a constraint that is either enforced or not. Structure is documented below.
        :param pulumi.Input[str] constraint: The name of the Constraint the Policy is configuring, for example, `serviceuser.services`. Check out the [complete list of available constraints](https://cloud.google.com/resource-manager/docs/organization-policy/understanding-constraints#available_constraints).
        :param pulumi.Input[str] etag: (Computed) The etag of the organization policy. `etag` is used for optimistic concurrency control as a way to help prevent simultaneous updates of a policy from overwriting each other.
        :param pulumi.Input[dict] list_policy: A policy that can define specific values that are allowed or denied for the given constraint. It can also be used to allow or deny all values. Structure is documented below.
        :param pulumi.Input[str] org_id: The numeric ID of the organization to set the policy for.
        :param pulumi.Input[dict] restore_policy: A restore policy is a constraint to restore the default policy. Structure is documented below.
        :param pulumi.Input[str] update_time: (Computed) The timestamp in RFC3339 UTC "Zulu" format, accurate to nanoseconds, representing when the variable was last updated. Example: "2016-10-09T12:33:37.578138407Z".
        :param pulumi.Input[float] version: Version of the Policy. Default version is 0.

        The **boolean_policy** object supports the following:

          * `enforced` (`pulumi.Input[bool]`) - If true, then the Policy is enforced. If false, then any configuration is acceptable.

        The **list_policy** object supports the following:

          * `allow` (`pulumi.Input[dict]`) - or `deny` - (Optional) One or the other must be set.
            * `all` (`pulumi.Input[bool]`) - The policy allows or denies all values.
            * `values` (`pulumi.Input[list]`) - The policy can define specific values that are allowed or denied.

          * `deny` (`pulumi.Input[dict]`)
            * `all` (`pulumi.Input[bool]`) - The policy allows or denies all values.
            * `values` (`pulumi.Input[list]`) - The policy can define specific values that are allowed or denied.

          * `inheritFromParent` (`pulumi.Input[bool]`) - If set to true, the values from the effective Policy of the parent resource
            are inherited, meaning the values set in this Policy are added to the values inherited up the hierarchy.
          * `suggestedValue` (`pulumi.Input[str]`) - The Google Cloud Console will try to default to a configuration that matches the value specified in this field.

        The **restore_policy** object supports the following:

          * `default` (`pulumi.Input[bool]`) - May only be set to true. If set, then the default Policy is restored.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["boolean_policy"] = boolean_policy
        __props__["constraint"] = constraint
        __props__["etag"] = etag
        __props__["list_policy"] = list_policy
        __props__["org_id"] = org_id
        __props__["restore_policy"] = restore_policy
        __props__["update_time"] = update_time
        __props__["version"] = version
        return Policy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class BackendBucket(pulumi.CustomResource):
    bucket_name: pulumi.Output[str]
    """
    Cloud Storage bucket name.
    """
    cdn_policy: pulumi.Output[dict]
    """
    Cloud CDN configuration for this Backend Bucket.  Structure is documented below.

      * `signedUrlCacheMaxAgeSec` (`float`) - Maximum number of seconds the response to a signed URL request will
        be considered fresh. After this time period,
        the response will be revalidated before being served.
        When serving responses to signed URL requests,
        Cloud CDN will internally behave as though
        all responses from this backend had a "Cache-Control: public,
        max-age=[TTL]" header, regardless of any existing Cache-Control
        header. The actual headers served in responses will not be altered.
    """
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional textual description of the resource; provided by the
    client when the resource is created.
    """
    enable_cdn: pulumi.Output[bool]
    """
    If true, enable Cloud CDN for this BackendBucket.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is
    created. The name must be 1-63 characters long, and comply with
    RFC1035.  Specifically, the name must be 1-63 characters long and
    match the regular expression `a-z?` which means
    the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the
    last character, which cannot be a dash.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, bucket_name=None, cdn_policy=None, description=None, enable_cdn=None, name=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        Backend buckets allow you to use Google Cloud Storage buckets with HTTP(S)
        load balancing.

        An HTTP(S) load balancer can direct traffic to specified URLs to a
        backend bucket rather than a backend service. It can send requests for
        static content to a Cloud Storage bucket and requests for dynamic content
        to a virtual machine instance.


        To get more information about BackendBucket, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/v1/backendBuckets)
        * How-to Guides
            * [Using a Cloud Storage bucket as a load balancer backend](https://cloud.google.com/compute/docs/load-balancing/http/backend-bucket)

        ## Example Usage

        ### Backend Bucket Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        image_bucket = gcp.storage.Bucket("imageBucket", location="EU")
        image_backend = gcp.compute.BackendBucket("imageBackend",
            description="Contains beautiful images",
            bucket_name=image_bucket.name,
            enable_cdn=True)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] bucket_name: Cloud Storage bucket name.
        :param pulumi.Input[dict] cdn_policy: Cloud CDN configuration for this Backend Bucket.  Structure is documented below.
        :param pulumi.Input[str] description: An optional textual description of the resource; provided by the
               client when the resource is created.
        :param pulumi.Input[bool] enable_cdn: If true, enable Cloud CDN for this BackendBucket.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the
               last character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **cdn_policy** object supports the following:

          * `signedUrlCacheMaxAgeSec` (`pulumi.Input[float]`) - Maximum number of seconds the response to a signed URL request will
            be considered fresh. After this time period,
            the response will be revalidated before being served.
            When serving responses to signed URL requests,
            Cloud CDN will internally behave as though
            all responses from this backend had a "Cache-Control: public,
            max-age=[TTL]" header, regardless of any existing Cache-Control
            header. The actual headers served in responses will not be altered.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if bucket_name is None:
                raise TypeError("Missing required property 'bucket_name'")
            __props__['bucket_name'] = bucket_name
            __props__['cdn_policy'] = cdn_policy
            __props__['description'] = description
            __props__['enable_cdn'] = enable_cdn
            __props__['name'] = name
            __props__['project'] = project
            __props__['creation_timestamp'] = None
            __props__['self_link'] = None
        super(BackendBucket, __self__).__init__(
            'gcp:compute/backendBucket:BackendBucket',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bucket_name=None, cdn_policy=None, creation_timestamp=None, description=None, enable_cdn=None, name=None, project=None, self_link=None):
        """
        Get an existing BackendBucket resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] bucket_name: Cloud Storage bucket name.
        :param pulumi.Input[dict] cdn_policy: Cloud CDN configuration for this Backend Bucket.  Structure is documented below.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional textual description of the resource; provided by the
               client when the resource is created.
        :param pulumi.Input[bool] enable_cdn: If true, enable Cloud CDN for this BackendBucket.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the
               last character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **cdn_policy** object supports the following:

          * `signedUrlCacheMaxAgeSec` (`pulumi.Input[float]`) - Maximum number of seconds the response to a signed URL request will
            be considered fresh. After this time period,
            the response will be revalidated before being served.
            When serving responses to signed URL requests,
            Cloud CDN will internally behave as though
            all responses from this backend had a "Cache-Control: public,
            max-age=[TTL]" header, regardless of any existing Cache-Control
            header. The actual headers served in responses will not be altered.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bucket_name"] = bucket_name
        __props__["cdn_policy"] = cdn_policy
        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["enable_cdn"] = enable_cdn
        __props__["name"] = name
        __props__["project"] = project
        __props__["self_link"] = self_link
        return BackendBucket(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Subnetwork(pulumi.CustomResource):
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional description of this resource. Provide this property when
    you create the resource. This field can be set only at resource
    creation time.
    """
    fingerprint: pulumi.Output[str]
    """
    Fingerprint of this resource. This field is used internally during updates of this resource.
    """
    gateway_address: pulumi.Output[str]
    """
    The gateway address for default routes to reach destination addresses outside this subnetwork.
    """
    ip_cidr_range: pulumi.Output[str]
    """
    The range of IP addresses belonging to this subnetwork secondary
    range. Provide this property when you create the subnetwork.
    Ranges must be unique and non-overlapping with all primary and
    secondary IP ranges within a network. Only IPv4 is supported.
    """
    log_config: pulumi.Output[dict]
    """
    Denotes the logging options for the subnetwork flow logs. If logging is enabled
    logs will be exported to Stackdriver. This field cannot be set if the `purpose` of this
    subnetwork is `INTERNAL_HTTPS_LOAD_BALANCER`  Structure is documented below.

      * `aggregationInterval` (`str`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
        Toggles the aggregation interval for collecting flow logs. Increasing the
        interval time will reduce the amount of generated flow logs for long
        lasting connections. Default is an interval of 5 seconds per connection.
        Possible values are INTERVAL_5_SEC, INTERVAL_30_SEC, INTERVAL_1_MIN,
        INTERVAL_5_MIN, INTERVAL_10_MIN, INTERVAL_15_MIN
      * `flowSampling` (`float`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
        The value of the field must be in [0, 1]. Set the sampling rate of VPC
        flow logs within the subnetwork where 1.0 means all collected logs are
        reported and 0.0 means no logs are reported. Default is 0.5 which means
        half of all collected logs are reported.
      * `metadata` (`str`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
        Configures whether metadata fields should be added to the reported VPC
        flow logs.
    """
    name: pulumi.Output[str]
    """
    The name of the resource, provided by the client when initially
    creating the resource. The name must be 1-63 characters long, and
    comply with RFC1035. Specifically, the name must be 1-63 characters
    long and match the regular expression `a-z?` which
    means the first character must be a lowercase letter, and all
    following characters must be a dash, lowercase letter, or digit,
    except the last character, which cannot be a dash.
    """
    network: pulumi.Output[str]
    """
    The network this subnet belongs to.
    Only networks that are in the distributed mode can have subnetworks.
    """
    private_ip_google_access: pulumi.Output[bool]
    """
    When enabled, VMs in this subnetwork without external IP addresses can
    access Google APIs and services by using Private Google Access.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    purpose: pulumi.Output[str]
    """
    The purpose of the resource. This field can be either PRIVATE
    or INTERNAL_HTTPS_LOAD_BALANCER. A subnetwork with purpose set to
    INTERNAL_HTTPS_LOAD_BALANCER is a user-created subnetwork that is
    reserved for Internal HTTP(S) Load Balancing. If unspecified, the
    purpose defaults to PRIVATE.
    If set to INTERNAL_HTTPS_LOAD_BALANCER you must also set the role.
    """
    region: pulumi.Output[str]
    """
    URL of the GCP region for this subnetwork.
    """
    role: pulumi.Output[str]
    """
    The role of subnetwork. Currently, this field is only used when
    purpose = INTERNAL_HTTPS_LOAD_BALANCER. The value can be set to ACTIVE
    or BACKUP. An ACTIVE subnetwork is one that is currently being used
    for Internal HTTP(S) Load Balancing. A BACKUP subnetwork is one that
    is ready to be promoted to ACTIVE or is currently draining.
    """
    secondary_ip_ranges: pulumi.Output[list]
    """
    An array of configurations for secondary IP ranges for VM instances
    contained in this subnetwork. The primary IP of such VM must belong
    to the primary ipCidrRange of the subnetwork. The alias IPs may belong
    to either primary or secondary ranges. Structure is documented below.

      * `ip_cidr_range` (`str`) - The range of IP addresses belonging to this subnetwork secondary
        range. Provide this property when you create the subnetwork.
        Ranges must be unique and non-overlapping with all primary and
        secondary IP ranges within a network. Only IPv4 is supported.
      * `rangeName` (`str`) - The name associated with this subnetwork secondary range, used
        when adding an alias IP range to a VM instance. The name must
        be 1-63 characters long, and comply with RFC1035. The name
        must be unique within the subnetwork.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, description=None, ip_cidr_range=None, log_config=None, name=None, network=None, private_ip_google_access=None, project=None, purpose=None, region=None, role=None, secondary_ip_ranges=None, __props__=None, __name__=None, __opts__=None):
        """
        A VPC network is a virtual version of the traditional physical networks
        that exist within and between physical data centers. A VPC network
        provides connectivity for your Compute Engine virtual machine (VM)
        instances, Container Engine containers, App Engine Flex services, and
        other network-related resources.

        Each GCP project contains one or more VPC networks. Each VPC network is a
        global entity spanning all GCP regions. This global VPC network allows VM
        instances and other resources to communicate with each other via internal,
        private IP addresses.

        Each VPC network is subdivided into subnets, and each subnet is contained
        within a single region. You can have more than one subnet in a region for
        a given VPC network. Each subnet has a contiguous private RFC1918 IP
        space. You create instances, containers, and the like in these subnets.
        When you create an instance, you must create it in a subnet, and the
        instance draws its internal IP address from that subnet.

        Virtual machine (VM) instances in a VPC network can communicate with
        instances in all other subnets of the same VPC network, regardless of
        region, using their RFC1918 private IP addresses. You can isolate portions
        of the network, even entire subnets, using firewall rules.


        To get more information about Subnetwork, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/beta/subnetworks)
        * How-to Guides
            * [Private Google Access](https://cloud.google.com/vpc/docs/configure-private-google-access)
            * [Cloud Networking](https://cloud.google.com/vpc/docs/using-vpc)

        ## Example Usage

        ### Subnetwork Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        custom_test = gcp.compute.Network("custom-test", auto_create_subnetworks=False)
        network_with_private_secondary_ip_ranges = gcp.compute.Subnetwork("network-with-private-secondary-ip-ranges",
            ip_cidr_range="10.2.0.0/16",
            region="us-central1",
            network=custom_test.id,
            secondary_ip_range=[{
                "rangeName": "tf-test-secondary-range-update1",
                "ip_cidr_range": "192.168.10.0/24",
            }])
        ```

        ### Subnetwork Logging Config

        ```python
        import pulumi
        import pulumi_gcp as gcp

        custom_test = gcp.compute.Network("custom-test", auto_create_subnetworks=False)
        subnet_with_logging = gcp.compute.Subnetwork("subnet-with-logging",
            ip_cidr_range="10.2.0.0/16",
            region="us-central1",
            network=custom_test.id,
            log_config={
                "aggregationInterval": "INTERVAL_10_MIN",
                "flowSampling": 0.5,
                "metadata": "INCLUDE_ALL_METADATA",
            })
        ```

        ### Subnetwork Internal L7lb

        ```python
        import pulumi
        import pulumi_gcp as gcp

        custom_test = gcp.compute.Network("custom-test", auto_create_subnetworks=False)
        network_for_l7lb = gcp.compute.Subnetwork("network-for-l7lb",
            ip_cidr_range="10.0.0.0/22",
            region="us-central1",
            purpose="INTERNAL_HTTPS_LOAD_BALANCER",
            role="ACTIVE",
            network=custom_test.id)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when
               you create the resource. This field can be set only at resource
               creation time.
        :param pulumi.Input[str] ip_cidr_range: The range of IP addresses belonging to this subnetwork secondary
               range. Provide this property when you create the subnetwork.
               Ranges must be unique and non-overlapping with all primary and
               secondary IP ranges within a network. Only IPv4 is supported.
        :param pulumi.Input[dict] log_config: Denotes the logging options for the subnetwork flow logs. If logging is enabled
               logs will be exported to Stackdriver. This field cannot be set if the `purpose` of this
               subnetwork is `INTERNAL_HTTPS_LOAD_BALANCER`  Structure is documented below.
        :param pulumi.Input[str] name: The name of the resource, provided by the client when initially
               creating the resource. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters
               long and match the regular expression `a-z?` which
               means the first character must be a lowercase letter, and all
               following characters must be a dash, lowercase letter, or digit,
               except the last character, which cannot be a dash.
        :param pulumi.Input[str] network: The network this subnet belongs to.
               Only networks that are in the distributed mode can have subnetworks.
        :param pulumi.Input[bool] private_ip_google_access: When enabled, VMs in this subnetwork without external IP addresses can
               access Google APIs and services by using Private Google Access.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] purpose: The purpose of the resource. This field can be either PRIVATE
               or INTERNAL_HTTPS_LOAD_BALANCER. A subnetwork with purpose set to
               INTERNAL_HTTPS_LOAD_BALANCER is a user-created subnetwork that is
               reserved for Internal HTTP(S) Load Balancing. If unspecified, the
               purpose defaults to PRIVATE.
               If set to INTERNAL_HTTPS_LOAD_BALANCER you must also set the role.
        :param pulumi.Input[str] region: URL of the GCP region for this subnetwork.
        :param pulumi.Input[str] role: The role of subnetwork. Currently, this field is only used when
               purpose = INTERNAL_HTTPS_LOAD_BALANCER. The value can be set to ACTIVE
               or BACKUP. An ACTIVE subnetwork is one that is currently being used
               for Internal HTTP(S) Load Balancing. A BACKUP subnetwork is one that
               is ready to be promoted to ACTIVE or is currently draining.
        :param pulumi.Input[list] secondary_ip_ranges: An array of configurations for secondary IP ranges for VM instances
               contained in this subnetwork. The primary IP of such VM must belong
               to the primary ipCidrRange of the subnetwork. The alias IPs may belong
               to either primary or secondary ranges. Structure is documented below.

        The **log_config** object supports the following:

          * `aggregationInterval` (`pulumi.Input[str]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            Toggles the aggregation interval for collecting flow logs. Increasing the
            interval time will reduce the amount of generated flow logs for long
            lasting connections. Default is an interval of 5 seconds per connection.
            Possible values are INTERVAL_5_SEC, INTERVAL_30_SEC, INTERVAL_1_MIN,
            INTERVAL_5_MIN, INTERVAL_10_MIN, INTERVAL_15_MIN
          * `flowSampling` (`pulumi.Input[float]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            The value of the field must be in [0, 1]. Set the sampling rate of VPC
            flow logs within the subnetwork where 1.0 means all collected logs are
            reported and 0.0 means no logs are reported. Default is 0.5 which means
            half of all collected logs are reported.
          * `metadata` (`pulumi.Input[str]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            Configures whether metadata fields should be added to the reported VPC
            flow logs.

        The **secondary_ip_ranges** object supports the following:

          * `ip_cidr_range` (`pulumi.Input[str]`) - The range of IP addresses belonging to this subnetwork secondary
            range. Provide this property when you create the subnetwork.
            Ranges must be unique and non-overlapping with all primary and
            secondary IP ranges within a network. Only IPv4 is supported.
          * `rangeName` (`pulumi.Input[str]`) - The name associated with this subnetwork secondary range, used
            when adding an alias IP range to a VM instance. The name must
            be 1-63 characters long, and comply with RFC1035. The name
            must be unique within the subnetwork.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            if ip_cidr_range is None:
                raise TypeError("Missing required property 'ip_cidr_range'")
            __props__['ip_cidr_range'] = ip_cidr_range
            __props__['log_config'] = log_config
            __props__['name'] = name
            if network is None:
                raise TypeError("Missing required property 'network'")
            __props__['network'] = network
            __props__['private_ip_google_access'] = private_ip_google_access
            __props__['project'] = project
            __props__['purpose'] = purpose
            __props__['region'] = region
            __props__['role'] = role
            __props__['secondary_ip_ranges'] = secondary_ip_ranges
            __props__['creation_timestamp'] = None
            __props__['fingerprint'] = None
            __props__['gateway_address'] = None
            __props__['self_link'] = None
        super(Subnetwork, __self__).__init__(
            'gcp:compute/subnetwork:Subnetwork',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, creation_timestamp=None, description=None, fingerprint=None, gateway_address=None, ip_cidr_range=None, log_config=None, name=None, network=None, private_ip_google_access=None, project=None, purpose=None, region=None, role=None, secondary_ip_ranges=None, self_link=None):
        """
        Get an existing Subnetwork resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when
               you create the resource. This field can be set only at resource
               creation time.
        :param pulumi.Input[str] fingerprint: Fingerprint of this resource. This field is used internally during updates of this resource.
        :param pulumi.Input[str] gateway_address: The gateway address for default routes to reach destination addresses outside this subnetwork.
        :param pulumi.Input[str] ip_cidr_range: The range of IP addresses belonging to this subnetwork secondary
               range. Provide this property when you create the subnetwork.
               Ranges must be unique and non-overlapping with all primary and
               secondary IP ranges within a network. Only IPv4 is supported.
        :param pulumi.Input[dict] log_config: Denotes the logging options for the subnetwork flow logs. If logging is enabled
               logs will be exported to Stackdriver. This field cannot be set if the `purpose` of this
               subnetwork is `INTERNAL_HTTPS_LOAD_BALANCER`  Structure is documented below.
        :param pulumi.Input[str] name: The name of the resource, provided by the client when initially
               creating the resource. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters
               long and match the regular expression `a-z?` which
               means the first character must be a lowercase letter, and all
               following characters must be a dash, lowercase letter, or digit,
               except the last character, which cannot be a dash.
        :param pulumi.Input[str] network: The network this subnet belongs to.
               Only networks that are in the distributed mode can have subnetworks.
        :param pulumi.Input[bool] private_ip_google_access: When enabled, VMs in this subnetwork without external IP addresses can
               access Google APIs and services by using Private Google Access.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] purpose: The purpose of the resource. This field can be either PRIVATE
               or INTERNAL_HTTPS_LOAD_BALANCER. A subnetwork with purpose set to
               INTERNAL_HTTPS_LOAD_BALANCER is a user-created subnetwork that is
               reserved for Internal HTTP(S) Load Balancing. If unspecified, the
               purpose defaults to PRIVATE.
               If set to INTERNAL_HTTPS_LOAD_BALANCER you must also set the role.
        :param pulumi.Input[str] region: URL of the GCP region for this subnetwork.
        :param pulumi.Input[str] role: The role of subnetwork. Currently, this field is only used when
               purpose = INTERNAL_HTTPS_LOAD_BALANCER. The value can be set to ACTIVE
               or BACKUP. An ACTIVE subnetwork is one that is currently being used
               for Internal HTTP(S) Load Balancing. A BACKUP subnetwork is one that
               is ready to be promoted to ACTIVE or is currently draining.
        :param pulumi.Input[list] secondary_ip_ranges: An array of configurations for secondary IP ranges for VM instances
               contained in this subnetwork. The primary IP of such VM must belong
               to the primary ipCidrRange of the subnetwork. The alias IPs may belong
               to either primary or secondary ranges. Structure is documented below.
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **log_config** object supports the following:

          * `aggregationInterval` (`pulumi.Input[str]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            Toggles the aggregation interval for collecting flow logs. Increasing the
            interval time will reduce the amount of generated flow logs for long
            lasting connections. Default is an interval of 5 seconds per connection.
            Possible values are INTERVAL_5_SEC, INTERVAL_30_SEC, INTERVAL_1_MIN,
            INTERVAL_5_MIN, INTERVAL_10_MIN, INTERVAL_15_MIN
          * `flowSampling` (`pulumi.Input[float]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            The value of the field must be in [0, 1]. Set the sampling rate of VPC
            flow logs within the subnetwork where 1.0 means all collected logs are
            reported and 0.0 means no logs are reported. Default is 0.5 which means
            half of all collected logs are reported.
          * `metadata` (`pulumi.Input[str]`) - Can only be specified if VPC flow logging for this subnetwork is enabled.
            Configures whether metadata fields should be added to the reported VPC
            flow logs.

        The **secondary_ip_ranges** object supports the following:

          * `ip_cidr_range` (`pulumi.Input[str]`) - The range of IP addresses belonging to this subnetwork secondary
            range. Provide this property when you create the subnetwork.
            Ranges must be unique and non-overlapping with all primary and
            secondary IP ranges within a network. Only IPv4 is supported.
          * `rangeName` (`pulumi.Input[str]`) - The name associated with this subnetwork secondary range, used
            when adding an alias IP range to a VM instance. The name must
            be 1-63 characters long, and comply with RFC1035. The name
            must be unique within the subnetwork.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["fingerprint"] = fingerprint
        __props__["gateway_address"] = gateway_address
        __props__["ip_cidr_range"] = ip_cidr_range
        __props__["log_config"] = log_config
        __props__["name"] = name
        __props__["network"] = network
        __props__["private_ip_google_access"] = private_ip_google_access
        __props__["project"] = project
        __props__["purpose"] = purpose
        __props__["region"] = region
        __props__["role"] = role
        __props__["secondary_ip_ranges"] = secondary_ip_ranges
        __props__["self_link"] = self_link
        return Subnetwork(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Budget(pulumi.CustomResource):
    all_updates_rule: pulumi.Output[dict]
    """
    Defines notifications that are sent on every update to the
    billing account's spend, regardless of the thresholds defined
    using threshold rules.  Structure is documented below.

      * `pubsubTopic` (`str`) - The name of the Cloud Pub/Sub topic where budget related
        messages will be published, in the form
        projects/{project_id}/topics/{topic_id}. Updates are sent
        at regular intervals to the topic.
      * `schemaVersion` (`str`) - The schema version of the notification. Only "1.0" is
        accepted. It represents the JSON schema as defined in
        https://cloud.google.com/billing/docs/how-to/budgets#notification_format.
    """
    amount: pulumi.Output[dict]
    """
    The budgeted amount for each usage period.  Structure is documented below.

      * `specifiedAmount` (`dict`) - A specified amount to use as the budget. currencyCode is
        optional. If specified, it must match the currency of the
        billing account. The currencyCode is provided on output.  Structure is documented below.
        * `currencyCode` (`str`) - The 3-letter currency code defined in ISO 4217.
        * `nanos` (`float`) - Number of nano (10^-9) units of the amount.
          The value must be between -999,999,999 and +999,999,999
          inclusive. If units is positive, nanos must be positive or
          zero. If units is zero, nanos can be positive, zero, or
          negative. If units is negative, nanos must be negative or
          zero. For example $-1.75 is represented as units=-1 and
          nanos=-750,000,000.
        * `units` (`str`) - The whole units of the amount. For example if currencyCode
          is "USD", then 1 unit is one US dollar.
    """
    billing_account: pulumi.Output[str]
    """
    ID of the billing account to set a budget on.
    """
    budget_filter: pulumi.Output[dict]
    """
    Filters that define which resources are used to compute the actual
    spend against the budget.  Structure is documented below.

      * `creditTypesTreatment` (`str`) - Specifies how credits should be treated when determining spend
        for threshold calculations.
      * `projects` (`list`) - A set of projects of the form projects/{project_id},
        specifying that usage from only this set of projects should be
        included in the budget. If omitted, the report will include
        all usage for the billing account, regardless of which project
        the usage occurred on. Only zero or one project can be
        specified currently.
      * `services` (`list`) - A set of services of the form services/{service_id},
        specifying that usage from only this set of services should be
        included in the budget. If omitted, the report will include
        usage for all the services. The service names are available
        through the Catalog API:
        https://cloud.google.com/billing/v1/how-tos/catalog-api.
    """
    display_name: pulumi.Output[str]
    """
    User data for display name in UI. Must be <= 60 chars.
    """
    name: pulumi.Output[str]
    """
    Resource name of the budget. The resource name implies the scope of a budget. Values are of the form
    billingAccounts/{billingAccountId}/budgets/{budgetId}.
    """
    threshold_rules: pulumi.Output[list]
    """
    Rules that trigger alerts (notifications of thresholds being
    crossed) when spend exceeds the specified percentages of the
    budget.  Structure is documented below.

      * `spendBasis` (`str`) - The type of basis used to determine if spend has passed
        the threshold.
      * `thresholdPercent` (`float`) - Send an alert when this threshold is exceeded. This is a
        1.0-based percentage, so 0.5 = 50%. Must be >= 0.
    """
    def __init__(__self__, resource_name, opts=None, all_updates_rule=None, amount=None, billing_account=None, budget_filter=None, display_name=None, threshold_rules=None, __props__=None, __name__=None, __opts__=None):
        """
        Budget configuration for a billing account.

        To get more information about Budget, see:

        * [API documentation](https://cloud.google.com/billing/docs/reference/budget/rest/v1beta1/billingAccounts.budgets)
        * How-to Guides
            * [Creating a budget](https://cloud.google.com/billing/docs/how-to/budgets)

        ## Example Usage

        ### Billing Budget Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        account = gcp.organizations.get_billing_account(billing_account="000000-0000000-0000000-000000")
        budget = gcp.billing.Budget("budget",
            billing_account=account.id,
            display_name="Example Billing Budget",
            amount={
                "specified_amount": {
                    "currencyCode": "USD",
                    "units": "100000",
                },
            },
            threshold_rules=[{
                "thresholdPercent": 0.5,
            }])
        ```

        ### Billing Budget Filter

        ```python
        import pulumi
        import pulumi_gcp as gcp

        account = gcp.organizations.get_billing_account(billing_account="000000-0000000-0000000-000000")
        budget = gcp.billing.Budget("budget",
            billing_account=account.id,
            display_name="Example Billing Budget",
            budget_filter={
                "projects": ["projects/my-project-name"],
                "creditTypesTreatment": "EXCLUDE_ALL_CREDITS",
                "services": ["services/24E6-581D-38E5"],
            },
            amount={
                "specified_amount": {
                    "currencyCode": "USD",
                    "units": "100000",
                },
            },
            threshold_rules=[
                {
                    "thresholdPercent": 0.5,
                },
                {
                    "thresholdPercent": 0.9,
                    "spendBasis": "FORECASTED_SPEND",
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] all_updates_rule: Defines notifications that are sent on every update to the
               billing account's spend, regardless of the thresholds defined
               using threshold rules.  Structure is documented below.
        :param pulumi.Input[dict] amount: The budgeted amount for each usage period.  Structure is documented below.
        :param pulumi.Input[str] billing_account: ID of the billing account to set a budget on.
        :param pulumi.Input[dict] budget_filter: Filters that define which resources are used to compute the actual
               spend against the budget.  Structure is documented below.
        :param pulumi.Input[str] display_name: User data for display name in UI. Must be <= 60 chars.
        :param pulumi.Input[list] threshold_rules: Rules that trigger alerts (notifications of thresholds being
               crossed) when spend exceeds the specified percentages of the
               budget.  Structure is documented below.

        The **all_updates_rule** object supports the following:

          * `pubsubTopic` (`pulumi.Input[str]`) - The name of the Cloud Pub/Sub topic where budget related
            messages will be published, in the form
            projects/{project_id}/topics/{topic_id}. Updates are sent
            at regular intervals to the topic.
          * `schemaVersion` (`pulumi.Input[str]`) - The schema version of the notification. Only "1.0" is
            accepted. It represents the JSON schema as defined in
            https://cloud.google.com/billing/docs/how-to/budgets#notification_format.

        The **amount** object supports the following:

          * `specifiedAmount` (`pulumi.Input[dict]`) - A specified amount to use as the budget. currencyCode is
            optional. If specified, it must match the currency of the
            billing account. The currencyCode is provided on output.  Structure is documented below.
            * `currencyCode` (`pulumi.Input[str]`) - The 3-letter currency code defined in ISO 4217.
            * `nanos` (`pulumi.Input[float]`) - Number of nano (10^-9) units of the amount.
              The value must be between -999,999,999 and +999,999,999
              inclusive. If units is positive, nanos must be positive or
              zero. If units is zero, nanos can be positive, zero, or
              negative. If units is negative, nanos must be negative or
              zero. For example $-1.75 is represented as units=-1 and
              nanos=-750,000,000.
            * `units` (`pulumi.Input[str]`) - The whole units of the amount. For example if currencyCode
              is "USD", then 1 unit is one US dollar.

        The **budget_filter** object supports the following:

          * `creditTypesTreatment` (`pulumi.Input[str]`) - Specifies how credits should be treated when determining spend
            for threshold calculations.
          * `projects` (`pulumi.Input[list]`) - A set of projects of the form projects/{project_id},
            specifying that usage from only this set of projects should be
            included in the budget. If omitted, the report will include
            all usage for the billing account, regardless of which project
            the usage occurred on. Only zero or one project can be
            specified currently.
          * `services` (`pulumi.Input[list]`) - A set of services of the form services/{service_id},
            specifying that usage from only this set of services should be
            included in the budget. If omitted, the report will include
            usage for all the services. The service names are available
            through the Catalog API:
            https://cloud.google.com/billing/v1/how-tos/catalog-api.

        The **threshold_rules** object supports the following:

          * `spendBasis` (`pulumi.Input[str]`) - The type of basis used to determine if spend has passed
            the threshold.
          * `thresholdPercent` (`pulumi.Input[float]`) - Send an alert when this threshold is exceeded. This is a
            1.0-based percentage, so 0.5 = 50%. Must be >= 0.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['all_updates_rule'] = all_updates_rule
            if amount is None:
                raise TypeError("Missing required property 'amount'")
            __props__['amount'] = amount
            if billing_account is None:
                raise TypeError("Missing required property 'billing_account'")
            __props__['billing_account'] = billing_account
            __props__['budget_filter'] = budget_filter
            __props__['display_name'] = display_name
            if threshold_rules is None:
                raise TypeError("Missing required property 'threshold_rules'")
            __props__['threshold_rules'] = threshold_rules
            __props__['name'] = None
        super(Budget, __self__).__init__(
            'gcp:billing/budget:Budget',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, all_updates_rule=None, amount=None, billing_account=None, budget_filter=None, display_name=None, name=None, threshold_rules=None):
        """
        Get an existing Budget resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] all_updates_rule: Defines notifications that are sent on every update to the
               billing account's spend, regardless of the thresholds defined
               using threshold rules.  Structure is documented below.
        :param pulumi.Input[dict] amount: The budgeted amount for each usage period.  Structure is documented below.
        :param pulumi.Input[str] billing_account: ID of the billing account to set a budget on.
        :param pulumi.Input[dict] budget_filter: Filters that define which resources are used to compute the actual
               spend against the budget.  Structure is documented below.
        :param pulumi.Input[str] display_name: User data for display name in UI. Must be <= 60 chars.
        :param pulumi.Input[str] name: Resource name of the budget. The resource name implies the scope of a budget. Values are of the form
               billingAccounts/{billingAccountId}/budgets/{budgetId}.
        :param pulumi.Input[list] threshold_rules: Rules that trigger alerts (notifications of thresholds being
               crossed) when spend exceeds the specified percentages of the
               budget.  Structure is documented below.

        The **all_updates_rule** object supports the following:

          * `pubsubTopic` (`pulumi.Input[str]`) - The name of the Cloud Pub/Sub topic where budget related
            messages will be published, in the form
            projects/{project_id}/topics/{topic_id}. Updates are sent
            at regular intervals to the topic.
          * `schemaVersion` (`pulumi.Input[str]`) - The schema version of the notification. Only "1.0" is
            accepted. It represents the JSON schema as defined in
            https://cloud.google.com/billing/docs/how-to/budgets#notification_format.

        The **amount** object supports the following:

          * `specifiedAmount` (`pulumi.Input[dict]`) - A specified amount to use as the budget. currencyCode is
            optional. If specified, it must match the currency of the
            billing account. The currencyCode is provided on output.  Structure is documented below.
            * `currencyCode` (`pulumi.Input[str]`) - The 3-letter currency code defined in ISO 4217.
            * `nanos` (`pulumi.Input[float]`) - Number of nano (10^-9) units of the amount.
              The value must be between -999,999,999 and +999,999,999
              inclusive. If units is positive, nanos must be positive or
              zero. If units is zero, nanos can be positive, zero, or
              negative. If units is negative, nanos must be negative or
              zero. For example $-1.75 is represented as units=-1 and
              nanos=-750,000,000.
            * `units` (`pulumi.Input[str]`) - The whole units of the amount. For example if currencyCode
              is "USD", then 1 unit is one US dollar.

        The **budget_filter** object supports the following:

          * `creditTypesTreatment` (`pulumi.Input[str]`) - Specifies how credits should be treated when determining spend
            for threshold calculations.
          * `projects` (`pulumi.Input[list]`) - A set of projects of the form projects/{project_id},
            specifying that usage from only this set of projects should be
            included in the budget. If omitted, the report will include
            all usage for the billing account, regardless of which project
            the usage occurred on. Only zero or one project can be
            specified currently.
          * `services` (`pulumi.Input[list]`) - A set of services of the form services/{service_id},
            specifying that usage from only this set of services should be
            included in the budget. If omitted, the report will include
            usage for all the services. The service names are available
            through the Catalog API:
            https://cloud.google.com/billing/v1/how-tos/catalog-api.

        The **threshold_rules** object supports the following:

          * `spendBasis` (`pulumi.Input[str]`) - The type of basis used to determine if spend has passed
            the threshold.
          * `thresholdPercent` (`pulumi.Input[float]`) - Send an alert when this threshold is exceeded. This is a
            1.0-based percentage, so 0.5 = 50%. Must be >= 0.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["all_updates_rule"] = all_updates_rule
        __props__["amount"] = amount
        __props__["billing_account"] = billing_account
        __props__["budget_filter"] = budget_filter
        __props__["display_name"] = display_name
        __props__["name"] = name
        __props__["threshold_rules"] = threshold_rules
        return Budget(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class AlertPolicy(pulumi.CustomResource):
    combiner: pulumi.Output[str]
    """
    How to combine the results of multiple conditions to
    determine if an incident should be opened.
    """
    conditions: pulumi.Output[list]
    """
    A list of conditions for the policy. The conditions are combined by
    AND or OR according to the combiner field. If the combined conditions
    evaluate to true, then an incident is created. A policy can have from
    one to six conditions.  Structure is documented below.

      * `conditionAbsent` (`dict`) - A condition that checks that a time series
        continues to receive new data points.  Structure is documented below.
        * `aggregations` (`list`) - Specifies the alignment of data points in
          individual time series as well as how to
          combine the retrieved time series together
          (such as when aggregating multiple streams
          on each resource to a single stream for each
          resource or when aggregating streams across
          all members of a group of resources).
          Multiple aggregations are applied in the
          order specified.This field is similar to the
          one in the MetricService.ListTimeSeries
          request. It is advisable to use the
          ListTimeSeries method when debugging this
          field.  Structure is documented below.
          * `alignmentPeriod` (`str`) - The alignment period for per-time
            series alignment. If present,
            alignmentPeriod must be at least
            60 seconds. After per-time series
            alignment, each time series will
            contain data points only on the
            period boundaries. If
            perSeriesAligner is not specified
            or equals ALIGN_NONE, then this
            field is ignored. If
            perSeriesAligner is specified and
            does not equal ALIGN_NONE, then
            this field must be defined;
            otherwise an error is returned.
          * `crossSeriesReducer` (`str`) - The approach to be used to combine
            time series. Not all reducer
            functions may be applied to all
            time series, depending on the
            metric type and the value type of
            the original time series.
            Reduction may change the metric
            type of value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.
          * `groupByFields` (`list`) - The set of fields to preserve when
            crossSeriesReducer is specified.
            The groupByFields determine how
            the time series are partitioned
            into subsets prior to applying the
            aggregation function. Each subset
            contains time series that have the
            same value for each of the
            grouping fields. Each individual
            time series is a member of exactly
            one subset. The crossSeriesReducer
            is applied to each subset of time
            series. It is not possible to
            reduce across different resource
            types, so this field implicitly
            contains resource.type. Fields not
            specified in groupByFields are
            aggregated away. If groupByFields
            is not specified and all the time
            series have the same resource
            type, then the time series are
            aggregated into a single output
            time series. If crossSeriesReducer
            is not defined, this field is
            ignored.
          * `perSeriesAligner` (`str`) - The approach to be used to align
            individual time series. Not all
            alignment functions may be applied
            to all time series, depending on
            the metric type and value type of
            the original time series.
            Alignment may change the metric
            type or the value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.

        * `duration` (`str`) - The amount of time that a time series must
          violate the threshold to be considered
          failing. Currently, only values that are a
          multiple of a minute--e.g., 0, 60, 120, or
          300 seconds--are supported. If an invalid
          value is given, an error will be returned.
          When choosing a duration, it is useful to
          keep in mind the frequency of the underlying
          time series data (which may also be affected
          by any alignments specified in the
          aggregations field); a good duration is long
          enough so that a single outlier does not
          generate spurious alerts, but short enough
          that unhealthy states are detected and
          alerted on quickly.
        * `filter` (`str`) - A filter that identifies which time series
          should be compared with the threshold.The
          filter is similar to the one that is
          specified in the
          MetricService.ListTimeSeries request (that
          call is useful to verify the time series
          that will be retrieved / processed) and must
          specify the metric type and optionally may
          contain restrictions on resource type,
          resource labels, and metric labels. This
          field may not exceed 2048 Unicode characters
          in length.
        * `trigger` (`dict`) - The number/percent of time series for which
          the comparison must hold in order for the
          condition to trigger. If unspecified, then
          the condition will trigger if the comparison
          is true for any of the time series that have
          been identified by filter and aggregations,
          or by the ratio, if denominator_filter and
          denominator_aggregations are specified.  Structure is documented below.
          * `count` (`float`) - The absolute number of time series
            that must fail the predicate for the
            condition to be triggered.
          * `percent` (`float`) - The percentage of time series that
            must fail the predicate for the
            condition to be triggered.

      * `conditionThreshold` (`dict`) - A condition that compares a time series against a
        threshold.  Structure is documented below.
        * `aggregations` (`list`) - Specifies the alignment of data points in
          individual time series as well as how to
          combine the retrieved time series together
          (such as when aggregating multiple streams
          on each resource to a single stream for each
          resource or when aggregating streams across
          all members of a group of resources).
          Multiple aggregations are applied in the
          order specified.This field is similar to the
          one in the MetricService.ListTimeSeries
          request. It is advisable to use the
          ListTimeSeries method when debugging this
          field.  Structure is documented below.
          * `alignmentPeriod` (`str`) - The alignment period for per-time
            series alignment. If present,
            alignmentPeriod must be at least
            60 seconds. After per-time series
            alignment, each time series will
            contain data points only on the
            period boundaries. If
            perSeriesAligner is not specified
            or equals ALIGN_NONE, then this
            field is ignored. If
            perSeriesAligner is specified and
            does not equal ALIGN_NONE, then
            this field must be defined;
            otherwise an error is returned.
          * `crossSeriesReducer` (`str`) - The approach to be used to combine
            time series. Not all reducer
            functions may be applied to all
            time series, depending on the
            metric type and the value type of
            the original time series.
            Reduction may change the metric
            type of value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.
          * `groupByFields` (`list`) - The set of fields to preserve when
            crossSeriesReducer is specified.
            The groupByFields determine how
            the time series are partitioned
            into subsets prior to applying the
            aggregation function. Each subset
            contains time series that have the
            same value for each of the
            grouping fields. Each individual
            time series is a member of exactly
            one subset. The crossSeriesReducer
            is applied to each subset of time
            series. It is not possible to
            reduce across different resource
            types, so this field implicitly
            contains resource.type. Fields not
            specified in groupByFields are
            aggregated away. If groupByFields
            is not specified and all the time
            series have the same resource
            type, then the time series are
            aggregated into a single output
            time series. If crossSeriesReducer
            is not defined, this field is
            ignored.
          * `perSeriesAligner` (`str`) - The approach to be used to align
            individual time series. Not all
            alignment functions may be applied
            to all time series, depending on
            the metric type and value type of
            the original time series.
            Alignment may change the metric
            type or the value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.

        * `comparison` (`str`) - The comparison to apply between the time
          series (indicated by filter and aggregation)
          and the threshold (indicated by
          threshold_value). The comparison is applied
          on each time series, with the time series on
          the left-hand side and the threshold on the
          right-hand side. Only COMPARISON_LT and
          COMPARISON_GT are supported currently.
        * `denominatorAggregations` (`list`) - Specifies the alignment of data points in
          individual time series selected by
          denominatorFilter as well as how to combine
          the retrieved time series together (such as
          when aggregating multiple streams on each
          resource to a single stream for each
          resource or when aggregating streams across
          all members of a group of resources).When
          computing ratios, the aggregations and
          denominator_aggregations fields must use the
          same alignment period and produce time
          series that have the same periodicity and
          labels.This field is similar to the one in
          the MetricService.ListTimeSeries request. It
          is advisable to use the ListTimeSeries
          method when debugging this field.  Structure is documented below.
          * `alignmentPeriod` (`str`) - The alignment period for per-time
            series alignment. If present,
            alignmentPeriod must be at least
            60 seconds. After per-time series
            alignment, each time series will
            contain data points only on the
            period boundaries. If
            perSeriesAligner is not specified
            or equals ALIGN_NONE, then this
            field is ignored. If
            perSeriesAligner is specified and
            does not equal ALIGN_NONE, then
            this field must be defined;
            otherwise an error is returned.
          * `crossSeriesReducer` (`str`) - The approach to be used to combine
            time series. Not all reducer
            functions may be applied to all
            time series, depending on the
            metric type and the value type of
            the original time series.
            Reduction may change the metric
            type of value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.
          * `groupByFields` (`list`) - The set of fields to preserve when
            crossSeriesReducer is specified.
            The groupByFields determine how
            the time series are partitioned
            into subsets prior to applying the
            aggregation function. Each subset
            contains time series that have the
            same value for each of the
            grouping fields. Each individual
            time series is a member of exactly
            one subset. The crossSeriesReducer
            is applied to each subset of time
            series. It is not possible to
            reduce across different resource
            types, so this field implicitly
            contains resource.type. Fields not
            specified in groupByFields are
            aggregated away. If groupByFields
            is not specified and all the time
            series have the same resource
            type, then the time series are
            aggregated into a single output
            time series. If crossSeriesReducer
            is not defined, this field is
            ignored.
          * `perSeriesAligner` (`str`) - The approach to be used to align
            individual time series. Not all
            alignment functions may be applied
            to all time series, depending on
            the metric type and value type of
            the original time series.
            Alignment may change the metric
            type or the value type of the time
            series.Time series data must be
            aligned in order to perform cross-
            time series reduction. If
            crossSeriesReducer is specified,
            then perSeriesAligner must be
            specified and not equal ALIGN_NONE
            and alignmentPeriod must be
            specified; otherwise, an error is
            returned.

        * `denominatorFilter` (`str`) - A filter that identifies a time series that
          should be used as the denominator of a ratio
          that will be compared with the threshold. If
          a denominator_filter is specified, the time
          series specified by the filter field will be
          used as the numerator.The filter is similar
          to the one that is specified in the
          MetricService.ListTimeSeries request (that
          call is useful to verify the time series
          that will be retrieved / processed) and must
          specify the metric type and optionally may
          contain restrictions on resource type,
          resource labels, and metric labels. This
          field may not exceed 2048 Unicode characters
          in length.
        * `duration` (`str`) - The amount of time that a time series must
          violate the threshold to be considered
          failing. Currently, only values that are a
          multiple of a minute--e.g., 0, 60, 120, or
          300 seconds--are supported. If an invalid
          value is given, an error will be returned.
          When choosing a duration, it is useful to
          keep in mind the frequency of the underlying
          time series data (which may also be affected
          by any alignments specified in the
          aggregations field); a good duration is long
          enough so that a single outlier does not
          generate spurious alerts, but short enough
          that unhealthy states are detected and
          alerted on quickly.
        * `filter` (`str`) - A filter that identifies which time series
          should be compared with the threshold.The
          filter is similar to the one that is
          specified in the
          MetricService.ListTimeSeries request (that
          call is useful to verify the time series
          that will be retrieved / processed) and must
          specify the metric type and optionally may
          contain restrictions on resource type,
          resource labels, and metric labels. This
          field may not exceed 2048 Unicode characters
          in length.
        * `thresholdValue` (`float`) - A value against which to compare the time
          series.
        * `trigger` (`dict`) - The number/percent of time series for which
          the comparison must hold in order for the
          condition to trigger. If unspecified, then
          the condition will trigger if the comparison
          is true for any of the time series that have
          been identified by filter and aggregations,
          or by the ratio, if denominator_filter and
          denominator_aggregations are specified.  Structure is documented below.
          * `count` (`float`) - The absolute number of time series
            that must fail the predicate for the
            condition to be triggered.
          * `percent` (`float`) - The percentage of time series that
            must fail the predicate for the
            condition to be triggered.

      * `display_name` (`str`) - A short name or phrase used to identify the
        condition in dashboards, notifications, and
        incidents. To avoid confusion, don't use the same
        display name for multiple conditions in the same
        policy.
      * `name` (`str`) - -
        The unique resource name for this condition.
        Its syntax is:
        projects/[PROJECT_ID]/alertPolicies/[POLICY_ID]/conditions/[CONDITION_ID]
        [CONDITION_ID] is assigned by Stackdriver Monitoring when
        the condition is created as part of a new or updated alerting
        policy.
    """
    creation_record: pulumi.Output[dict]
    """
    A read-only record of the creation of the alerting policy. If provided in a call to create or update, this field will be
    ignored.

      * `mutateTime` (`str`)
      * `mutatedBy` (`str`)
    """
    display_name: pulumi.Output[str]
    """
    A short name or phrase used to identify the
    condition in dashboards, notifications, and
    incidents. To avoid confusion, don't use the same
    display name for multiple conditions in the same
    policy.
    """
    documentation: pulumi.Output[dict]
    """
    A short name or phrase used to identify the policy in dashboards,
    notifications, and incidents. To avoid confusion, don't use the same
    display name for multiple policies in the same project. The name is
    limited to 512 Unicode characters.  Structure is documented below.

      * `content` (`str`) - The text of the documentation, interpreted according to mimeType.
        The content may not exceed 8,192 Unicode characters and may not
        exceed more than 10,240 bytes when encoded in UTF-8 format,
        whichever is smaller.
      * `mimeType` (`str`) - The format of the content field. Presently, only the value
        "text/markdown" is supported.
    """
    enabled: pulumi.Output[bool]
    """
    Whether or not the policy is enabled. The default is true.
    """
    name: pulumi.Output[str]
    """
    -
    The unique resource name for this condition.
    Its syntax is:
    projects/[PROJECT_ID]/alertPolicies/[POLICY_ID]/conditions/[CONDITION_ID]
    [CONDITION_ID] is assigned by Stackdriver Monitoring when
    the condition is created as part of a new or updated alerting
    policy.
    """
    notification_channels: pulumi.Output[list]
    """
    Identifies the notification channels to which notifications should be
    sent when incidents are opened or closed or when new violations occur
    on an already opened incident. Each element of this array corresponds
    to the name field in each of the NotificationChannel objects that are
    returned from the notificationChannels.list method. The syntax of the
    entries in this field is
    `projects/[PROJECT_ID]/notificationChannels/[CHANNEL_ID]`
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    user_labels: pulumi.Output[dict]
    """
    This field is intended to be used for organizing and identifying the AlertPolicy
    objects.The field can contain up to 64 entries. Each key and value is limited
    to 63 Unicode characters or 128 bytes, whichever is smaller. Labels and values
    can contain only lowercase letters, numerals, underscores, and dashes. Keys
    must begin with a letter.
    """
    def __init__(__self__, resource_name, opts=None, combiner=None, conditions=None, display_name=None, documentation=None, enabled=None, notification_channels=None, project=None, user_labels=None, __props__=None, __name__=None, __opts__=None):
        """
        A description of the conditions under which some aspect of your system is
        considered to be "unhealthy" and the ways to notify people or services
        about this state.


        To get more information about AlertPolicy, see:

        * [API documentation](https://cloud.google.com/monitoring/api/ref_v3/rest/v3/projects.alertPolicies)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/monitoring/alerts/)

        ## Example Usage

        ### Monitoring Alert Policy Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        alert_policy = gcp.monitoring.AlertPolicy("alertPolicy",
            combiner="OR",
            conditions=[{
                "conditionThreshold": {
                    "aggregations": [{
                        "alignmentPeriod": "60s",
                        "perSeriesAligner": "ALIGN_RATE",
                    }],
                    "comparison": "COMPARISON_GT",
                    "duration": "60s",
                    "filter": "metric.type=\"compute.googleapis.com/instance/disk/write_bytes_count\" AND resource.type=\"gce_instance\"",
                },
                "display_name": "test condition",
            }],
            display_name="My Alert Policy",
            user_labels={
                "foo": "bar",
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] combiner: How to combine the results of multiple conditions to
               determine if an incident should be opened.
        :param pulumi.Input[list] conditions: A list of conditions for the policy. The conditions are combined by
               AND or OR according to the combiner field. If the combined conditions
               evaluate to true, then an incident is created. A policy can have from
               one to six conditions.  Structure is documented below.
        :param pulumi.Input[str] display_name: A short name or phrase used to identify the
               condition in dashboards, notifications, and
               incidents. To avoid confusion, don't use the same
               display name for multiple conditions in the same
               policy.
        :param pulumi.Input[dict] documentation: A short name or phrase used to identify the policy in dashboards,
               notifications, and incidents. To avoid confusion, don't use the same
               display name for multiple policies in the same project. The name is
               limited to 512 Unicode characters.  Structure is documented below.
        :param pulumi.Input[bool] enabled: Whether or not the policy is enabled. The default is true.
        :param pulumi.Input[list] notification_channels: Identifies the notification channels to which notifications should be
               sent when incidents are opened or closed or when new violations occur
               on an already opened incident. Each element of this array corresponds
               to the name field in each of the NotificationChannel objects that are
               returned from the notificationChannels.list method. The syntax of the
               entries in this field is
               `projects/[PROJECT_ID]/notificationChannels/[CHANNEL_ID]`
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] user_labels: This field is intended to be used for organizing and identifying the AlertPolicy
               objects.The field can contain up to 64 entries. Each key and value is limited
               to 63 Unicode characters or 128 bytes, whichever is smaller. Labels and values
               can contain only lowercase letters, numerals, underscores, and dashes. Keys
               must begin with a letter.

        The **conditions** object supports the following:

          * `conditionAbsent` (`pulumi.Input[dict]`) - A condition that checks that a time series
            continues to receive new data points.  Structure is documented below.
            * `aggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series as well as how to
              combine the retrieved time series together
              (such as when aggregating multiple streams
              on each resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).
              Multiple aggregations are applied in the
              order specified.This field is similar to the
              one in the MetricService.ListTimeSeries
              request. It is advisable to use the
              ListTimeSeries method when debugging this
              field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `duration` (`pulumi.Input[str]`) - The amount of time that a time series must
              violate the threshold to be considered
              failing. Currently, only values that are a
              multiple of a minute--e.g., 0, 60, 120, or
              300 seconds--are supported. If an invalid
              value is given, an error will be returned.
              When choosing a duration, it is useful to
              keep in mind the frequency of the underlying
              time series data (which may also be affected
              by any alignments specified in the
              aggregations field); a good duration is long
              enough so that a single outlier does not
              generate spurious alerts, but short enough
              that unhealthy states are detected and
              alerted on quickly.
            * `filter` (`pulumi.Input[str]`) - A filter that identifies which time series
              should be compared with the threshold.The
              filter is similar to the one that is
              specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `trigger` (`pulumi.Input[dict]`) - The number/percent of time series for which
              the comparison must hold in order for the
              condition to trigger. If unspecified, then
              the condition will trigger if the comparison
              is true for any of the time series that have
              been identified by filter and aggregations,
              or by the ratio, if denominator_filter and
              denominator_aggregations are specified.  Structure is documented below.
              * `count` (`pulumi.Input[float]`) - The absolute number of time series
                that must fail the predicate for the
                condition to be triggered.
              * `percent` (`pulumi.Input[float]`) - The percentage of time series that
                must fail the predicate for the
                condition to be triggered.

          * `conditionThreshold` (`pulumi.Input[dict]`) - A condition that compares a time series against a
            threshold.  Structure is documented below.
            * `aggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series as well as how to
              combine the retrieved time series together
              (such as when aggregating multiple streams
              on each resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).
              Multiple aggregations are applied in the
              order specified.This field is similar to the
              one in the MetricService.ListTimeSeries
              request. It is advisable to use the
              ListTimeSeries method when debugging this
              field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `comparison` (`pulumi.Input[str]`) - The comparison to apply between the time
              series (indicated by filter and aggregation)
              and the threshold (indicated by
              threshold_value). The comparison is applied
              on each time series, with the time series on
              the left-hand side and the threshold on the
              right-hand side. Only COMPARISON_LT and
              COMPARISON_GT are supported currently.
            * `denominatorAggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series selected by
              denominatorFilter as well as how to combine
              the retrieved time series together (such as
              when aggregating multiple streams on each
              resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).When
              computing ratios, the aggregations and
              denominator_aggregations fields must use the
              same alignment period and produce time
              series that have the same periodicity and
              labels.This field is similar to the one in
              the MetricService.ListTimeSeries request. It
              is advisable to use the ListTimeSeries
              method when debugging this field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `denominatorFilter` (`pulumi.Input[str]`) - A filter that identifies a time series that
              should be used as the denominator of a ratio
              that will be compared with the threshold. If
              a denominator_filter is specified, the time
              series specified by the filter field will be
              used as the numerator.The filter is similar
              to the one that is specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `duration` (`pulumi.Input[str]`) - The amount of time that a time series must
              violate the threshold to be considered
              failing. Currently, only values that are a
              multiple of a minute--e.g., 0, 60, 120, or
              300 seconds--are supported. If an invalid
              value is given, an error will be returned.
              When choosing a duration, it is useful to
              keep in mind the frequency of the underlying
              time series data (which may also be affected
              by any alignments specified in the
              aggregations field); a good duration is long
              enough so that a single outlier does not
              generate spurious alerts, but short enough
              that unhealthy states are detected and
              alerted on quickly.
            * `filter` (`pulumi.Input[str]`) - A filter that identifies which time series
              should be compared with the threshold.The
              filter is similar to the one that is
              specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `thresholdValue` (`pulumi.Input[float]`) - A value against which to compare the time
              series.
            * `trigger` (`pulumi.Input[dict]`) - The number/percent of time series for which
              the comparison must hold in order for the
              condition to trigger. If unspecified, then
              the condition will trigger if the comparison
              is true for any of the time series that have
              been identified by filter and aggregations,
              or by the ratio, if denominator_filter and
              denominator_aggregations are specified.  Structure is documented below.
              * `count` (`pulumi.Input[float]`) - The absolute number of time series
                that must fail the predicate for the
                condition to be triggered.
              * `percent` (`pulumi.Input[float]`) - The percentage of time series that
                must fail the predicate for the
                condition to be triggered.

          * `display_name` (`pulumi.Input[str]`) - A short name or phrase used to identify the
            condition in dashboards, notifications, and
            incidents. To avoid confusion, don't use the same
            display name for multiple conditions in the same
            policy.
          * `name` (`pulumi.Input[str]`) - -
            The unique resource name for this condition.
            Its syntax is:
            projects/[PROJECT_ID]/alertPolicies/[POLICY_ID]/conditions/[CONDITION_ID]
            [CONDITION_ID] is assigned by Stackdriver Monitoring when
            the condition is created as part of a new or updated alerting
            policy.

        The **documentation** object supports the following:

          * `content` (`pulumi.Input[str]`) - The text of the documentation, interpreted according to mimeType.
            The content may not exceed 8,192 Unicode characters and may not
            exceed more than 10,240 bytes when encoded in UTF-8 format,
            whichever is smaller.
          * `mimeType` (`pulumi.Input[str]`) - The format of the content field. Presently, only the value
            "text/markdown" is supported.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if combiner is None:
                raise TypeError("Missing required property 'combiner'")
            __props__['combiner'] = combiner
            if conditions is None:
                raise TypeError("Missing required property 'conditions'")
            __props__['conditions'] = conditions
            if display_name is None:
                raise TypeError("Missing required property 'display_name'")
            __props__['display_name'] = display_name
            __props__['documentation'] = documentation
            __props__['enabled'] = enabled
            __props__['notification_channels'] = notification_channels
            __props__['project'] = project
            __props__['user_labels'] = user_labels
            __props__['creation_record'] = None
            __props__['name'] = None
        super(AlertPolicy, __self__).__init__(
            'gcp:monitoring/alertPolicy:AlertPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, combiner=None, conditions=None, creation_record=None, display_name=None, documentation=None, enabled=None, name=None, notification_channels=None, project=None, user_labels=None):
        """
        Get an existing AlertPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] combiner: How to combine the results of multiple conditions to
               determine if an incident should be opened.
        :param pulumi.Input[list] conditions: A list of conditions for the policy. The conditions are combined by
               AND or OR according to the combiner field. If the combined conditions
               evaluate to true, then an incident is created. A policy can have from
               one to six conditions.  Structure is documented below.
        :param pulumi.Input[dict] creation_record: A read-only record of the creation of the alerting policy. If provided in a call to create or update, this field will be
               ignored.
        :param pulumi.Input[str] display_name: A short name or phrase used to identify the
               condition in dashboards, notifications, and
               incidents. To avoid confusion, don't use the same
               display name for multiple conditions in the same
               policy.
        :param pulumi.Input[dict] documentation: A short name or phrase used to identify the policy in dashboards,
               notifications, and incidents. To avoid confusion, don't use the same
               display name for multiple policies in the same project. The name is
               limited to 512 Unicode characters.  Structure is documented below.
        :param pulumi.Input[bool] enabled: Whether or not the policy is enabled. The default is true.
        :param pulumi.Input[str] name: -
               The unique resource name for this condition.
               Its syntax is:
               projects/[PROJECT_ID]/alertPolicies/[POLICY_ID]/conditions/[CONDITION_ID]
               [CONDITION_ID] is assigned by Stackdriver Monitoring when
               the condition is created as part of a new or updated alerting
               policy.
        :param pulumi.Input[list] notification_channels: Identifies the notification channels to which notifications should be
               sent when incidents are opened or closed or when new violations occur
               on an already opened incident. Each element of this array corresponds
               to the name field in each of the NotificationChannel objects that are
               returned from the notificationChannels.list method. The syntax of the
               entries in this field is
               `projects/[PROJECT_ID]/notificationChannels/[CHANNEL_ID]`
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] user_labels: This field is intended to be used for organizing and identifying the AlertPolicy
               objects.The field can contain up to 64 entries. Each key and value is limited
               to 63 Unicode characters or 128 bytes, whichever is smaller. Labels and values
               can contain only lowercase letters, numerals, underscores, and dashes. Keys
               must begin with a letter.

        The **conditions** object supports the following:

          * `conditionAbsent` (`pulumi.Input[dict]`) - A condition that checks that a time series
            continues to receive new data points.  Structure is documented below.
            * `aggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series as well as how to
              combine the retrieved time series together
              (such as when aggregating multiple streams
              on each resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).
              Multiple aggregations are applied in the
              order specified.This field is similar to the
              one in the MetricService.ListTimeSeries
              request. It is advisable to use the
              ListTimeSeries method when debugging this
              field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `duration` (`pulumi.Input[str]`) - The amount of time that a time series must
              violate the threshold to be considered
              failing. Currently, only values that are a
              multiple of a minute--e.g., 0, 60, 120, or
              300 seconds--are supported. If an invalid
              value is given, an error will be returned.
              When choosing a duration, it is useful to
              keep in mind the frequency of the underlying
              time series data (which may also be affected
              by any alignments specified in the
              aggregations field); a good duration is long
              enough so that a single outlier does not
              generate spurious alerts, but short enough
              that unhealthy states are detected and
              alerted on quickly.
            * `filter` (`pulumi.Input[str]`) - A filter that identifies which time series
              should be compared with the threshold.The
              filter is similar to the one that is
              specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `trigger` (`pulumi.Input[dict]`) - The number/percent of time series for which
              the comparison must hold in order for the
              condition to trigger. If unspecified, then
              the condition will trigger if the comparison
              is true for any of the time series that have
              been identified by filter and aggregations,
              or by the ratio, if denominator_filter and
              denominator_aggregations are specified.  Structure is documented below.
              * `count` (`pulumi.Input[float]`) - The absolute number of time series
                that must fail the predicate for the
                condition to be triggered.
              * `percent` (`pulumi.Input[float]`) - The percentage of time series that
                must fail the predicate for the
                condition to be triggered.

          * `conditionThreshold` (`pulumi.Input[dict]`) - A condition that compares a time series against a
            threshold.  Structure is documented below.
            * `aggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series as well as how to
              combine the retrieved time series together
              (such as when aggregating multiple streams
              on each resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).
              Multiple aggregations are applied in the
              order specified.This field is similar to the
              one in the MetricService.ListTimeSeries
              request. It is advisable to use the
              ListTimeSeries method when debugging this
              field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `comparison` (`pulumi.Input[str]`) - The comparison to apply between the time
              series (indicated by filter and aggregation)
              and the threshold (indicated by
              threshold_value). The comparison is applied
              on each time series, with the time series on
              the left-hand side and the threshold on the
              right-hand side. Only COMPARISON_LT and
              COMPARISON_GT are supported currently.
            * `denominatorAggregations` (`pulumi.Input[list]`) - Specifies the alignment of data points in
              individual time series selected by
              denominatorFilter as well as how to combine
              the retrieved time series together (such as
              when aggregating multiple streams on each
              resource to a single stream for each
              resource or when aggregating streams across
              all members of a group of resources).When
              computing ratios, the aggregations and
              denominator_aggregations fields must use the
              same alignment period and produce time
              series that have the same periodicity and
              labels.This field is similar to the one in
              the MetricService.ListTimeSeries request. It
              is advisable to use the ListTimeSeries
              method when debugging this field.  Structure is documented below.
              * `alignmentPeriod` (`pulumi.Input[str]`) - The alignment period for per-time
                series alignment. If present,
                alignmentPeriod must be at least
                60 seconds. After per-time series
                alignment, each time series will
                contain data points only on the
                period boundaries. If
                perSeriesAligner is not specified
                or equals ALIGN_NONE, then this
                field is ignored. If
                perSeriesAligner is specified and
                does not equal ALIGN_NONE, then
                this field must be defined;
                otherwise an error is returned.
              * `crossSeriesReducer` (`pulumi.Input[str]`) - The approach to be used to combine
                time series. Not all reducer
                functions may be applied to all
                time series, depending on the
                metric type and the value type of
                the original time series.
                Reduction may change the metric
                type of value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.
              * `groupByFields` (`pulumi.Input[list]`) - The set of fields to preserve when
                crossSeriesReducer is specified.
                The groupByFields determine how
                the time series are partitioned
                into subsets prior to applying the
                aggregation function. Each subset
                contains time series that have the
                same value for each of the
                grouping fields. Each individual
                time series is a member of exactly
                one subset. The crossSeriesReducer
                is applied to each subset of time
                series. It is not possible to
                reduce across different resource
                types, so this field implicitly
                contains resource.type. Fields not
                specified in groupByFields are
                aggregated away. If groupByFields
                is not specified and all the time
                series have the same resource
                type, then the time series are
                aggregated into a single output
                time series. If crossSeriesReducer
                is not defined, this field is
                ignored.
              * `perSeriesAligner` (`pulumi.Input[str]`) - The approach to be used to align
                individual time series. Not all
                alignment functions may be applied
                to all time series, depending on
                the metric type and value type of
                the original time series.
                Alignment may change the metric
                type or the value type of the time
                series.Time series data must be
                aligned in order to perform cross-
                time series reduction. If
                crossSeriesReducer is specified,
                then perSeriesAligner must be
                specified and not equal ALIGN_NONE
                and alignmentPeriod must be
                specified; otherwise, an error is
                returned.

            * `denominatorFilter` (`pulumi.Input[str]`) - A filter that identifies a time series that
              should be used as the denominator of a ratio
              that will be compared with the threshold. If
              a denominator_filter is specified, the time
              series specified by the filter field will be
              used as the numerator.The filter is similar
              to the one that is specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `duration` (`pulumi.Input[str]`) - The amount of time that a time series must
              violate the threshold to be considered
              failing. Currently, only values that are a
              multiple of a minute--e.g., 0, 60, 120, or
              300 seconds--are supported. If an invalid
              value is given, an error will be returned.
              When choosing a duration, it is useful to
              keep in mind the frequency of the underlying
              time series data (which may also be affected
              by any alignments specified in the
              aggregations field); a good duration is long
              enough so that a single outlier does not
              generate spurious alerts, but short enough
              that unhealthy states are detected and
              alerted on quickly.
            * `filter` (`pulumi.Input[str]`) - A filter that identifies which time series
              should be compared with the threshold.The
              filter is similar to the one that is
              specified in the
              MetricService.ListTimeSeries request (that
              call is useful to verify the time series
              that will be retrieved / processed) and must
              specify the metric type and optionally may
              contain restrictions on resource type,
              resource labels, and metric labels. This
              field may not exceed 2048 Unicode characters
              in length.
            * `thresholdValue` (`pulumi.Input[float]`) - A value against which to compare the time
              series.
            * `trigger` (`pulumi.Input[dict]`) - The number/percent of time series for which
              the comparison must hold in order for the
              condition to trigger. If unspecified, then
              the condition will trigger if the comparison
              is true for any of the time series that have
              been identified by filter and aggregations,
              or by the ratio, if denominator_filter and
              denominator_aggregations are specified.  Structure is documented below.
              * `count` (`pulumi.Input[float]`) - The absolute number of time series
                that must fail the predicate for the
                condition to be triggered.
              * `percent` (`pulumi.Input[float]`) - The percentage of time series that
                must fail the predicate for the
                condition to be triggered.

          * `display_name` (`pulumi.Input[str]`) - A short name or phrase used to identify the
            condition in dashboards, notifications, and
            incidents. To avoid confusion, don't use the same
            display name for multiple conditions in the same
            policy.
          * `name` (`pulumi.Input[str]`) - -
            The unique resource name for this condition.
            Its syntax is:
            projects/[PROJECT_ID]/alertPolicies/[POLICY_ID]/conditions/[CONDITION_ID]
            [CONDITION_ID] is assigned by Stackdriver Monitoring when
            the condition is created as part of a new or updated alerting
            policy.

        The **creation_record** object supports the following:

          * `mutateTime` (`pulumi.Input[str]`)
          * `mutatedBy` (`pulumi.Input[str]`)

        The **documentation** object supports the following:

          * `content` (`pulumi.Input[str]`) - The text of the documentation, interpreted according to mimeType.
            The content may not exceed 8,192 Unicode characters and may not
            exceed more than 10,240 bytes when encoded in UTF-8 format,
            whichever is smaller.
          * `mimeType` (`pulumi.Input[str]`) - The format of the content field. Presently, only the value
            "text/markdown" is supported.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["combiner"] = combiner
        __props__["conditions"] = conditions
        __props__["creation_record"] = creation_record
        __props__["display_name"] = display_name
        __props__["documentation"] = documentation
        __props__["enabled"] = enabled
        __props__["name"] = name
        __props__["notification_channels"] = notification_channels
        __props__["project"] = project
        __props__["user_labels"] = user_labels
        return AlertPolicy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class ApplicationUrlDispatchRules(pulumi.CustomResource):
    dispatch_rules: pulumi.Output[list]
    """
    Rules to match an HTTP request and dispatch that request to a service.  Structure is documented below.

      * `domain` (`str`) - Domain name to match against. The wildcard "*" is supported if specified before a period: "*.".
        Defaults to matching all domains: "*".
      * `path` (`str`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
        The sum of the lengths of the domain and path may not exceed 100 characters.
      * `service` (`str`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
        The sum of the lengths of the domain and path may not exceed 100 characters.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, dispatch_rules=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        Rules to match an HTTP request and dispatch that request to a service.


        To get more information about ApplicationUrlDispatchRules, see:

        * [API documentation](https://cloud.google.com/appengine/docs/admin-api/reference/rest/v1/apps#UrlDispatchRule)

        ## Example Usage

        ### App Engine Application Url Dispatch Rules Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bucket = gcp.storage.Bucket("bucket")
        object = gcp.storage.BucketObject("object",
            bucket=bucket.name,
            source=pulumi.FileAsset("./test-fixtures/appengine/hello-world.zip"))
        admin_v3 = gcp.appengine.StandardAppVersion("adminV3",
            version_id="v3",
            service="admin",
            runtime="nodejs10",
            entrypoint={
                "shell": "node ./app.js",
            },
            deployment={
                "zip": {
                    "sourceUrl": pulumi.Output.all(bucket.name, object.name).apply(lambda bucketName, objectName: f"https://storage.googleapis.com/{bucket_name}/{object_name}"),
                },
            },
            env_variables={
                "port": "8080",
            },
            noop_on_destroy=True)
        web_service = gcp.appengine.ApplicationUrlDispatchRules("webService", dispatch_rules=[
            {
                "domain": "*",
                "path": "/*",
                "service": "default",
            },
            {
                "domain": "*",
                "path": "/admin/*",
                "service": admin_v3.service,
            },
        ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] dispatch_rules: Rules to match an HTTP request and dispatch that request to a service.  Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **dispatch_rules** object supports the following:

          * `domain` (`pulumi.Input[str]`) - Domain name to match against. The wildcard "*" is supported if specified before a period: "*.".
            Defaults to matching all domains: "*".
          * `path` (`pulumi.Input[str]`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
            The sum of the lengths of the domain and path may not exceed 100 characters.
          * `service` (`pulumi.Input[str]`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
            The sum of the lengths of the domain and path may not exceed 100 characters.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if dispatch_rules is None:
                raise TypeError("Missing required property 'dispatch_rules'")
            __props__['dispatch_rules'] = dispatch_rules
            __props__['project'] = project
        super(ApplicationUrlDispatchRules, __self__).__init__(
            'gcp:appengine/applicationUrlDispatchRules:ApplicationUrlDispatchRules',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, dispatch_rules=None, project=None):
        """
        Get an existing ApplicationUrlDispatchRules resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] dispatch_rules: Rules to match an HTTP request and dispatch that request to a service.  Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **dispatch_rules** object supports the following:

          * `domain` (`pulumi.Input[str]`) - Domain name to match against. The wildcard "*" is supported if specified before a period: "*.".
            Defaults to matching all domains: "*".
          * `path` (`pulumi.Input[str]`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
            The sum of the lengths of the domain and path may not exceed 100 characters.
          * `service` (`pulumi.Input[str]`) - Pathname within the host. Must start with a "/". A single "*" can be included at the end of the path.
            The sum of the lengths of the domain and path may not exceed 100 characters.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["dispatch_rules"] = dispatch_rules
        __props__["project"] = project
        return ApplicationUrlDispatchRules(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Project(pulumi.CustomResource):
    auto_create_network: pulumi.Output[bool]
    """
    Create the 'default' network automatically.  Default `true`.
    If set to `false`, the default network will be deleted.  Note that, for quota purposes, you
    will still need to have 1 network slot available to create the project successfully, even if
    you set `auto_create_network` to `false`, since the network will exist momentarily.
    """
    billing_account: pulumi.Output[str]
    """
    The alphanumeric ID of the billing account this project
    belongs to. The user or service account performing this operation with the provider
    must have Billing Account Administrator privileges (`roles/billing.admin`) in
    the organization. See [Google Cloud Billing API Access Control](https://cloud.google.com/billing/v1/how-tos/access-control)
    for more details.
    """
    folder_id: pulumi.Output[str]
    """
    The numeric ID of the folder this project should be
    created under. Only one of `org_id` or `folder_id` may be
    specified. If the `folder_id` is specified, then the project is
    created under the specified folder. Changing this forces the
    project to be migrated to the newly specified folder.
    """
    labels: pulumi.Output[dict]
    """
    A set of key/value label pairs to assign to the project.
    """
    name: pulumi.Output[str]
    """
    The display name of the project.
    """
    number: pulumi.Output[str]
    """
    The numeric identifier of the project.
    """
    org_id: pulumi.Output[str]
    """
    The numeric ID of the organization this project belongs to.
    Changing this forces a new project to be created.  Only one of
    `org_id` or `folder_id` may be specified. If the `org_id` is
    specified then the project is created at the top level. Changing
    this forces the project to be migrated to the newly specified
    organization.
    """
    project_id: pulumi.Output[str]
    """
    The project ID. Changing this forces a new project to be created.
    """
    skip_delete: pulumi.Output[bool]
    """
    If true, the resource can be deleted
    without deleting the Project via the Google API.
    """
    def __init__(__self__, resource_name, opts=None, auto_create_network=None, billing_account=None, folder_id=None, labels=None, name=None, org_id=None, project_id=None, skip_delete=None, __props__=None, __name__=None, __opts__=None):
        """
        Allows creation and management of a Google Cloud Platform project.

        Projects created with this resource must be associated with an Organization.
        See the [Organization documentation](https://cloud.google.com/resource-manager/docs/quickstarts) for more details.

        The service account used to run this provider when creating a `organizations.Project`
        resource must have `roles/resourcemanager.projectCreator`. See the
        [Access Control for Organizations Using IAM](https://cloud.google.com/resource-manager/docs/access-control-org)
        doc for more information.


        ## Example Usage



        ```python
        import pulumi
        import pulumi_gcp as gcp

        my_project = gcp.organizations.Project("myProject",
            org_id="1234567",
            project_id="your-project-id")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] auto_create_network: Create the 'default' network automatically.  Default `true`.
               If set to `false`, the default network will be deleted.  Note that, for quota purposes, you
               will still need to have 1 network slot available to create the project successfully, even if
               you set `auto_create_network` to `false`, since the network will exist momentarily.
        :param pulumi.Input[str] billing_account: The alphanumeric ID of the billing account this project
               belongs to. The user or service account performing this operation with the provider
               must have Billing Account Administrator privileges (`roles/billing.admin`) in
               the organization. See [Google Cloud Billing API Access Control](https://cloud.google.com/billing/v1/how-tos/access-control)
               for more details.
        :param pulumi.Input[str] folder_id: The numeric ID of the folder this project should be
               created under. Only one of `org_id` or `folder_id` may be
               specified. If the `folder_id` is specified, then the project is
               created under the specified folder. Changing this forces the
               project to be migrated to the newly specified folder.
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to the project.
        :param pulumi.Input[str] name: The display name of the project.
        :param pulumi.Input[str] org_id: The numeric ID of the organization this project belongs to.
               Changing this forces a new project to be created.  Only one of
               `org_id` or `folder_id` may be specified. If the `org_id` is
               specified then the project is created at the top level. Changing
               this forces the project to be migrated to the newly specified
               organization.
        :param pulumi.Input[str] project_id: The project ID. Changing this forces a new project to be created.
        :param pulumi.Input[bool] skip_delete: If true, the resource can be deleted
               without deleting the Project via the Google API.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['auto_create_network'] = auto_create_network
            __props__['billing_account'] = billing_account
            __props__['folder_id'] = folder_id
            __props__['labels'] = labels
            __props__['name'] = name
            __props__['org_id'] = org_id
            if project_id is None:
                raise TypeError("Missing required property 'project_id'")
            __props__['project_id'] = project_id
            __props__['skip_delete'] = skip_delete
            __props__['number'] = None
        super(Project, __self__).__init__(
            'gcp:organizations/project:Project',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, auto_create_network=None, billing_account=None, folder_id=None, labels=None, name=None, number=None, org_id=None, project_id=None, skip_delete=None):
        """
        Get an existing Project resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] auto_create_network: Create the 'default' network automatically.  Default `true`.
               If set to `false`, the default network will be deleted.  Note that, for quota purposes, you
               will still need to have 1 network slot available to create the project successfully, even if
               you set `auto_create_network` to `false`, since the network will exist momentarily.
        :param pulumi.Input[str] billing_account: The alphanumeric ID of the billing account this project
               belongs to. The user or service account performing this operation with the provider
               must have Billing Account Administrator privileges (`roles/billing.admin`) in
               the organization. See [Google Cloud Billing API Access Control](https://cloud.google.com/billing/v1/how-tos/access-control)
               for more details.
        :param pulumi.Input[str] folder_id: The numeric ID of the folder this project should be
               created under. Only one of `org_id` or `folder_id` may be
               specified. If the `folder_id` is specified, then the project is
               created under the specified folder. Changing this forces the
               project to be migrated to the newly specified folder.
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to the project.
        :param pulumi.Input[str] name: The display name of the project.
        :param pulumi.Input[str] number: The numeric identifier of the project.
        :param pulumi.Input[str] org_id: The numeric ID of the organization this project belongs to.
               Changing this forces a new project to be created.  Only one of
               `org_id` or `folder_id` may be specified. If the `org_id` is
               specified then the project is created at the top level. Changing
               this forces the project to be migrated to the newly specified
               organization.
        :param pulumi.Input[str] project_id: The project ID. Changing this forces a new project to be created.
        :param pulumi.Input[bool] skip_delete: If true, the resource can be deleted
               without deleting the Project via the Google API.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["auto_create_network"] = auto_create_network
        __props__["billing_account"] = billing_account
        __props__["folder_id"] = folder_id
        __props__["labels"] = labels
        __props__["name"] = name
        __props__["number"] = number
        __props__["org_id"] = org_id
        __props__["project_id"] = project_id
        __props__["skip_delete"] = skip_delete
        return Project(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class ExternalVpnGateway(pulumi.CustomResource):
    description: pulumi.Output[str]
    """
    An optional description of this resource.
    """
    interfaces: pulumi.Output[list]
    """
    A list of interfaces on this external VPN gateway.  Structure is documented below.

      * `id` (`float`) - The numberic ID for this interface. Allowed values are based on the redundancy type
        of this external VPN gateway
        * `0 - SINGLE_IP_INTERNALLY_REDUNDANT`
        * `0, 1 - TWO_IPS_REDUNDANCY`
        * `0, 1, 2, 3 - FOUR_IPS_REDUNDANCY`
      * `ip_address` (`str`) - IP address of the interface in the external VPN gateway.
        Only IPv4 is supported. This IP address can be either from
        your on-premise gateway or another Cloud provider’s VPN gateway,
        it cannot be an IP address from Google Compute Engine.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is
    created. The name must be 1-63 characters long, and comply with
    RFC1035.  Specifically, the name must be 1-63 characters long and
    match the regular expression `a-z?` which means
    the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the last
    character, which cannot be a dash.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    redundancy_type: pulumi.Output[str]
    """
    Indicates the redundancy type of this external VPN gateway
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, description=None, interfaces=None, name=None, project=None, redundancy_type=None, __props__=None, __name__=None, __opts__=None):
        """
        Represents a VPN gateway managed outside of GCP.

        To get more information about ExternalVpnGateway, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/beta/externalVpnGateways)

        ## Example Usage

        ### External Vpn Gateway

        ```python
        import pulumi
        import pulumi_gcp as gcp

        network = gcp.compute.Network("network",
            routing_mode="GLOBAL",
            auto_create_subnetworks=False)
        ha_gateway = gcp.compute.HaVpnGateway("haGateway",
            region="us-central1",
            network=network.id)
        external_gateway = gcp.compute.ExternalVpnGateway("externalGateway",
            redundancy_type="SINGLE_IP_INTERNALLY_REDUNDANT",
            description="An externally managed VPN gateway",
            interface=[{
                "id": 0,
                "ip_address": "8.8.8.8",
            }])
        network_subnet1 = gcp.compute.Subnetwork("networkSubnet1",
            ip_cidr_range="10.0.1.0/24",
            region="us-central1",
            network=network.id)
        network_subnet2 = gcp.compute.Subnetwork("networkSubnet2",
            ip_cidr_range="10.0.2.0/24",
            region="us-west1",
            network=network.id)
        router1 = gcp.compute.Router("router1",
            network=network.name,
            bgp={
                "asn": 64514,
            })
        tunnel1 = gcp.compute.VPNTunnel("tunnel1",
            region="us-central1",
            vpn_gateway=ha_gateway.id,
            peer_external_gateway=external_gateway.id,
            peer_external_gateway_interface=0,
            shared_secret="a secret message",
            router=router1.id,
            vpn_gateway_interface=0)
        tunnel2 = gcp.compute.VPNTunnel("tunnel2",
            region="us-central1",
            vpn_gateway=ha_gateway.id,
            peer_external_gateway=external_gateway.id,
            peer_external_gateway_interface=0,
            shared_secret="a secret message",
            router=router1.id.apply(lambda id: f" {id}"),
            vpn_gateway_interface=1)
        router1_interface1 = gcp.compute.RouterInterface("router1Interface1",
            router=router1.name,
            region="us-central1",
            ip_range="169.254.0.1/30",
            vpn_tunnel=tunnel1.name)
        router1_peer1 = gcp.compute.RouterPeer("router1Peer1",
            router=router1.name,
            region="us-central1",
            peer_ip_address="169.254.0.2",
            peer_asn=64515,
            advertised_route_priority=100,
            interface=router1_interface1.name)
        router1_interface2 = gcp.compute.RouterInterface("router1Interface2",
            router=router1.name,
            region="us-central1",
            ip_range="169.254.1.1/30",
            vpn_tunnel=tunnel2.name)
        router1_peer2 = gcp.compute.RouterPeer("router1Peer2",
            router=router1.name,
            region="us-central1",
            peer_ip_address="169.254.1.2",
            peer_asn=64515,
            advertised_route_priority=100,
            interface=router1_interface2.name)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[list] interfaces: A list of interfaces on this external VPN gateway.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] redundancy_type: Indicates the redundancy type of this external VPN gateway

        The **interfaces** object supports the following:

          * `id` (`pulumi.Input[float]`) - The numberic ID for this interface. Allowed values are based on the redundancy type
            of this external VPN gateway
            * `0 - SINGLE_IP_INTERNALLY_REDUNDANT`
            * `0, 1 - TWO_IPS_REDUNDANCY`
            * `0, 1, 2, 3 - FOUR_IPS_REDUNDANCY`
          * `ip_address` (`pulumi.Input[str]`) - IP address of the interface in the external VPN gateway.
            Only IPv4 is supported. This IP address can be either from
            your on-premise gateway or another Cloud provider’s VPN gateway,
            it cannot be an IP address from Google Compute Engine.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['interfaces'] = interfaces
            __props__['name'] = name
            __props__['project'] = project
            __props__['redundancy_type'] = redundancy_type
            __props__['self_link'] = None
        super(ExternalVpnGateway, __self__).__init__(
            'gcp:compute/externalVpnGateway:ExternalVpnGateway',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, description=None, interfaces=None, name=None, project=None, redundancy_type=None, self_link=None):
        """
        Get an existing ExternalVpnGateway resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[list] interfaces: A list of interfaces on this external VPN gateway.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] redundancy_type: Indicates the redundancy type of this external VPN gateway
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **interfaces** object supports the following:

          * `id` (`pulumi.Input[float]`) - The numberic ID for this interface. Allowed values are based on the redundancy type
            of this external VPN gateway
            * `0 - SINGLE_IP_INTERNALLY_REDUNDANT`
            * `0, 1 - TWO_IPS_REDUNDANCY`
            * `0, 1, 2, 3 - FOUR_IPS_REDUNDANCY`
          * `ip_address` (`pulumi.Input[str]`) - IP address of the interface in the external VPN gateway.
            Only IPv4 is supported. This IP address can be either from
            your on-premise gateway or another Cloud provider’s VPN gateway,
            it cannot be an IP address from Google Compute Engine.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["description"] = description
        __props__["interfaces"] = interfaces
        __props__["name"] = name
        __props__["project"] = project
        __props__["redundancy_type"] = redundancy_type
        __props__["self_link"] = self_link
        return ExternalVpnGateway(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Intent(pulumi.CustomResource):
    action: pulumi.Output[str]
    """
    The name of the action associated with the intent.
    Note: The action name must not contain whitespaces.
    """
    default_response_platforms: pulumi.Output[list]
    """
    The list of platforms for which the first responses will be copied from the messages in PLATFORM_UNSPECIFIED
    (i.e. default platform).
    """
    display_name: pulumi.Output[str]
    """
    The name of this intent to be displayed on the console.
    """
    events: pulumi.Output[list]
    """
    The collection of event names that trigger the intent. If the collection of input contexts is not empty, all of
    the contexts must be present in the active user session for an event to trigger this intent. See the
    [events reference](https://cloud.google.com/dialogflow/docs/events-overview) for more details.
    """
    followup_intent_infos: pulumi.Output[list]
    """
    Information about all followup intents that have this intent as a direct or indirect parent. We populate this field only
    in the output.

      * `followupIntentName` (`str`)
      * `parent_followup_intent_name` (`str`) - The unique identifier of the parent intent in the chain of followup intents.
        Format: projects/<Project ID>/agent/intents/<Intent ID>.
    """
    input_context_names: pulumi.Output[list]
    """
    The list of context names required for this intent to be triggered.
    Format: projects/<Project ID>/agent/sessions/-/contexts/<Context ID>.
    """
    is_fallback: pulumi.Output[bool]
    """
    Indicates whether this is a fallback intent.
    """
    ml_disabled: pulumi.Output[bool]
    """
    Indicates whether Machine Learning is disabled for the intent.
    Note: If mlDisabled setting is set to true, then this intent is not taken into account during inference in ML
    ONLY match mode. Also, auto-markup in the UI is turned off.
    """
    name: pulumi.Output[str]
    """
    The unique identifier of this intent. Format: projects/<Project ID>/agent/intents/<Intent ID>.
    """
    parent_followup_intent_name: pulumi.Output[str]
    """
    The unique identifier of the parent intent in the chain of followup intents.
    Format: projects/<Project ID>/agent/intents/<Intent ID>.
    """
    priority: pulumi.Output[float]
    """
    The priority of this intent. Higher numbers represent higher priorities.
    - If the supplied value is unspecified or 0, the service translates the value to 500,000, which corresponds
    to the Normal priority in the console.
    - If the supplied value is negative, the intent is ignored in runtime detect intent requests.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    reset_contexts: pulumi.Output[bool]
    """
    Indicates whether to delete all contexts in the current session when this intent is matched.
    """
    root_followup_intent_name: pulumi.Output[str]
    """
    The unique identifier of the root intent in the chain of followup intents. It identifies the correct followup intents
    chain for this intent. Format: projects/<Project ID>/agent/intents/<Intent ID>.
    """
    webhook_state: pulumi.Output[str]
    """
    Indicates whether webhooks are enabled for the intent.
    * WEBHOOK_STATE_ENABLED: Webhook is enabled in the agent and in the intent.
    * WEBHOOK_STATE_ENABLED_FOR_SLOT_FILLING: Webhook is enabled in the agent and in the intent. Also, each slot
    filling prompt is forwarded to the webhook.
    """
    def __init__(__self__, resource_name, opts=None, action=None, default_response_platforms=None, display_name=None, events=None, input_context_names=None, is_fallback=None, ml_disabled=None, parent_followup_intent_name=None, priority=None, project=None, reset_contexts=None, webhook_state=None, __props__=None, __name__=None, __opts__=None):
        """
        Represents a Dialogflow intent. Intents convert a number of user expressions or patterns into an action. An action
        is an extraction of a user command or sentence semantics.


        To get more information about Intent, see:

        * [API documentation](https://cloud.google.com/dialogflow/docs/reference/rest/v2/projects.agent.intents)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/dialogflow/docs/)

        ## Example Usage

        ### Dialogflow Intent Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        basic_agent = gcp.diagflow.Agent("basicAgent",
            display_name="example_agent",
            default_language_code="en",
            time_zone="America/New_York")
        basic_intent = gcp.diagflow.Intent("basicIntent", display_name="basic-intent")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] action: The name of the action associated with the intent.
               Note: The action name must not contain whitespaces.
        :param pulumi.Input[list] default_response_platforms: The list of platforms for which the first responses will be copied from the messages in PLATFORM_UNSPECIFIED
               (i.e. default platform).
        :param pulumi.Input[str] display_name: The name of this intent to be displayed on the console.
        :param pulumi.Input[list] events: The collection of event names that trigger the intent. If the collection of input contexts is not empty, all of
               the contexts must be present in the active user session for an event to trigger this intent. See the
               [events reference](https://cloud.google.com/dialogflow/docs/events-overview) for more details.
        :param pulumi.Input[list] input_context_names: The list of context names required for this intent to be triggered.
               Format: projects/<Project ID>/agent/sessions/-/contexts/<Context ID>.
        :param pulumi.Input[bool] is_fallback: Indicates whether this is a fallback intent.
        :param pulumi.Input[bool] ml_disabled: Indicates whether Machine Learning is disabled for the intent.
               Note: If mlDisabled setting is set to true, then this intent is not taken into account during inference in ML
               ONLY match mode. Also, auto-markup in the UI is turned off.
        :param pulumi.Input[str] parent_followup_intent_name: The unique identifier of the parent intent in the chain of followup intents.
               Format: projects/<Project ID>/agent/intents/<Intent ID>.
        :param pulumi.Input[float] priority: The priority of this intent. Higher numbers represent higher priorities.
               - If the supplied value is unspecified or 0, the service translates the value to 500,000, which corresponds
               to the Normal priority in the console.
               - If the supplied value is negative, the intent is ignored in runtime detect intent requests.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[bool] reset_contexts: Indicates whether to delete all contexts in the current session when this intent is matched.
        :param pulumi.Input[str] webhook_state: Indicates whether webhooks are enabled for the intent.
               * WEBHOOK_STATE_ENABLED: Webhook is enabled in the agent and in the intent.
               * WEBHOOK_STATE_ENABLED_FOR_SLOT_FILLING: Webhook is enabled in the agent and in the intent. Also, each slot
               filling prompt is forwarded to the webhook.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['action'] = action
            __props__['default_response_platforms'] = default_response_platforms
            if display_name is None:
                raise TypeError("Missing required property 'display_name'")
            __props__['display_name'] = display_name
            __props__['events'] = events
            __props__['input_context_names'] = input_context_names
            __props__['is_fallback'] = is_fallback
            __props__['ml_disabled'] = ml_disabled
            __props__['parent_followup_intent_name'] = parent_followup_intent_name
            __props__['priority'] = priority
            __props__['project'] = project
            __props__['reset_contexts'] = reset_contexts
            __props__['webhook_state'] = webhook_state
            __props__['followup_intent_infos'] = None
            __props__['name'] = None
            __props__['root_followup_intent_name'] = None
        super(Intent, __self__).__init__(
            'gcp:diagflow/intent:Intent',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, action=None, default_response_platforms=None, display_name=None, events=None, followup_intent_infos=None, input_context_names=None, is_fallback=None, ml_disabled=None, name=None, parent_followup_intent_name=None, priority=None, project=None, reset_contexts=None, root_followup_intent_name=None, webhook_state=None):
        """
        Get an existing Intent resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] action: The name of the action associated with the intent.
               Note: The action name must not contain whitespaces.
        :param pulumi.Input[list] default_response_platforms: The list of platforms for which the first responses will be copied from the messages in PLATFORM_UNSPECIFIED
               (i.e. default platform).
        :param pulumi.Input[str] display_name: The name of this intent to be displayed on the console.
        :param pulumi.Input[list] events: The collection of event names that trigger the intent. If the collection of input contexts is not empty, all of
               the contexts must be present in the active user session for an event to trigger this intent. See the
               [events reference](https://cloud.google.com/dialogflow/docs/events-overview) for more details.
        :param pulumi.Input[list] followup_intent_infos: Information about all followup intents that have this intent as a direct or indirect parent. We populate this field only
               in the output.
        :param pulumi.Input[list] input_context_names: The list of context names required for this intent to be triggered.
               Format: projects/<Project ID>/agent/sessions/-/contexts/<Context ID>.
        :param pulumi.Input[bool] is_fallback: Indicates whether this is a fallback intent.
        :param pulumi.Input[bool] ml_disabled: Indicates whether Machine Learning is disabled for the intent.
               Note: If mlDisabled setting is set to true, then this intent is not taken into account during inference in ML
               ONLY match mode. Also, auto-markup in the UI is turned off.
        :param pulumi.Input[str] name: The unique identifier of this intent. Format: projects/<Project ID>/agent/intents/<Intent ID>.
        :param pulumi.Input[str] parent_followup_intent_name: The unique identifier of the parent intent in the chain of followup intents.
               Format: projects/<Project ID>/agent/intents/<Intent ID>.
        :param pulumi.Input[float] priority: The priority of this intent. Higher numbers represent higher priorities.
               - If the supplied value is unspecified or 0, the service translates the value to 500,000, which corresponds
               to the Normal priority in the console.
               - If the supplied value is negative, the intent is ignored in runtime detect intent requests.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[bool] reset_contexts: Indicates whether to delete all contexts in the current session when this intent is matched.
        :param pulumi.Input[str] root_followup_intent_name: The unique identifier of the root intent in the chain of followup intents. It identifies the correct followup intents
               chain for this intent. Format: projects/<Project ID>/agent/intents/<Intent ID>.
        :param pulumi.Input[str] webhook_state: Indicates whether webhooks are enabled for the intent.
               * WEBHOOK_STATE_ENABLED: Webhook is enabled in the agent and in the intent.
               * WEBHOOK_STATE_ENABLED_FOR_SLOT_FILLING: Webhook is enabled in the agent and in the intent. Also, each slot
               filling prompt is forwarded to the webhook.

        The **followup_intent_infos** object supports the following:

          * `followupIntentName` (`pulumi.Input[str]`)
          * `parent_followup_intent_name` (`pulumi.Input[str]`) - The unique identifier of the parent intent in the chain of followup intents.
            Format: projects/<Project ID>/agent/intents/<Intent ID>.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["action"] = action
        __props__["default_response_platforms"] = default_response_platforms
        __props__["display_name"] = display_name
        __props__["events"] = events
        __props__["followup_intent_infos"] = followup_intent_infos
        __props__["input_context_names"] = input_context_names
        __props__["is_fallback"] = is_fallback
        __props__["ml_disabled"] = ml_disabled
        __props__["name"] = name
        __props__["parent_followup_intent_name"] = parent_followup_intent_name
        __props__["priority"] = priority
        __props__["project"] = project
        __props__["reset_contexts"] = reset_contexts
        __props__["root_followup_intent_name"] = root_followup_intent_name
        __props__["webhook_state"] = webhook_state
        return Intent(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

